<?php

define('WE_COLLECTION_ITEMS_LIMIT', 10);

class DbCollectionItemAdmin extends DbCollectionItem
{
	
	/* Gets collectionitems from a specific collection limited by offset and range
	 * @offset: start of the resultset
	 * @count: number of occurrences
	 * @collection: the Collection object
	 * returns: an array of CollectionItem objects or an empty array */
	public function GetByOffset($offset, $count, Collection $collection)
	{
		$items = array();
		$query = $collection->has_description ?
			'select ci.*, NULL as content1 from collectionitems as ci
			where collection_id = ' . $collection->id . '
			order by date_created desc
			limit ' . $offset . ', ' . $this->escape($count)
			:
			'SELECT ci.*, cc.content1 from collectionitems as ci, collectionitems_content as cc
			where ci.collection_id = ' . $collection->id . ' and ci.id = cc.item_id
			order by ci.date_created desc limit ' . $offset . ', ' . $this->escape($count);
		if ($result = $this->mysqli->query($query))
		{
			while ($obj = $result->fetch_object())
			{
				$item = $this->map($obj);
				$item->content1 = $obj->content1;
				$items[] = $item;
			}
			$result->close();
		}
		return $items;
	}
	
	/* Inserts a new CollectionItem
	 * @item: the CollectionItem to insert
	 * returns: the ID of the created CollectionItem or 0 on error */
	public function Create(CollectionItem $item)
	{
		if (!$this->mysqli->query(
			'insert into collectionitems set title = \'' . $this->escape($item->title) . '\'
			, url_title = \'' . $this->escape($item->url_title) . '\'
			, description = \'' . $this->escape($item->description) . '\'
			, collection_id = ' . $this->escape($item->collection_id) . '
			, image = \'' . $this->escape($item->image) . '\', comments_closed = ' . $item->comments_closed . '
			, visible = ' . $item->visible . ', meta_description = \'' . $this->escape($item->meta_description) . '\'' .
			', meta_keywords = \'' . $this->escape($item->meta_keywords) . '\''))
			{
				return 0;
			}

		$item->id = $this->mysqli->insert_id;
		
		$this->mysqli->query(
			'insert into collectionitems_content set item_id = ' . $item->id . ', content1 = \'\', content2 = \'\'');
		
		return $item->id;
	}

	/* Deletes CollectionItem with ID $id
	 * @id: the ID of the CollectionItem
	 * returns: true on succes, false on failure */
	public function Delete($id)
	{
		return (
			$this->mysqli->query('delete from collectionitems_tags where item_id = ' . $id) &&
			$this->mysqli->query('delete from collectionitems_content where item_id = ' . $id) &&
			$this->mysqli->query('delete from collectionitems_comments where item_id = ' . $id) &&
			$this->mysqli->query('delete from collectionitems where id = ' . $id));
	}
	
	public function update_basic($id, $itemTitle, $visible)
	{
		return $this->mysqli->query('update collectionitems set visible = ' . $visible . ', title = \'' . $this->escape($itemTitle) . '\'' .
			' where id = ' . $id);
	}
	
	public function update_seo($id, $url, $meta_keywords, $meta_description)
	{
		return $this->mysqli->query('update collectionitems set url_title = \'' . $this->escape($url) . '\'' .
			', meta_keywords = \'' . $this->escape($meta_keywords) . '\', meta_description = \'' . $this->escape($meta_description) . '\' where id = ' . $id);
	}
	
	public function update_comments_closed($id, $commentsClosed)
	{
		return $this->mysqli->query('update collectionitems set comments_closed = ' . $commentsClosed . ' where id = ' . $id);
	}
	
	public function update_image($id, $image)
	{
		return $this->mysqli->query('update collectionitems set image = \'' . $this->escape($image) . '\' where id = ' . $id);
	}
	
	public function update_description($id, $description)
	{
		return $this->mysqli->query('update collectionitems set description = \'' . $this->escape($description) . '\' where id = ' . $id);
	}
	
	/* Updates the content and visiblility of the  CollectionItem with ID $id
	 * @id: the id of the CollectionItem that should be updated
	 * @content: the updated content of the item
	 * @visible: the updated visibility
	 * returns: true on succes, false on error */
	public function UpdateContentAndVisible($id, $content1, $content2, $visible)
	{
		return (
			$this->mysqli->query('update collectionitems set visible = ' . $visible . ' where id = ' . $id) &&
			$this->mysqli->query(
				'update collectionitems_content set content1 = \'' . $this->escape($content1) . '\', content2 = \'' . $this->escape($content2) . '\' where item_id = ' . $id));
	}
}

class DbCollectionAdmin extends DbCollection
{
	/* Gets all Collection objects
	 * returns: array of Collection objects or an empty array */
	public function GetAll()
	{
		$types = array();
		if ($result = $this->mysqli->query(
			'select c.*, count(distinct cic.id) as not_accepted_comments_count, count(distinct ci.id) as items_count
			from (collections as c)
			left join collectionitems as ci on c.id = ci.collection_id
			left join collectionitems_comments as cic on (cic.item_id = ci.id and cic.accepted = 0)
			group by c.id order by c.name'))
		{
			while ($obj = $result->fetch_object())
			{
				$item = $this->map($obj);
				$item->not_accepted_comments_count = $obj->not_accepted_comments_count;
				$item->items_count = $obj->items_count;
				$types[] = $item;
			}
			$result->close();
		}
		return $types;
	}
	
	/* Creates a new Collection $c
	 * @c: the Collection to create
	 * returns: the ID of the newly created Collection or 0 on error */
	public function Create(Collection $c)
	{
		if ($result = $this->mysqli->query(
			'insert into collections set `name` = \'' . $this->escape($c->name) . '\',
			`has_description` = ' . $this->escape($c->has_description) . '
			, comments_add_email = \'' . $this->escape($c->comments_add_email) . '\'
			, has_image = ' . $this->escape($c->has_image) . '
			, has_content = ' . $this->escape($c->has_content) . '
			, has_comments = ' . $this->escape($c->has_comments) . '
			, comments_accepted_default = ' . $this->escape($c->comments_accepted_default)))
		{
			return $this->mysqli->insert_id;
		}
		return 0;
	}
	
	/* Updates the Collection with ID $id with values of $collection
	 * @c: the Collection with the new values
	 * @id: the ID of the Collection to update
	 * returns: true on success, false on failure */
	public function Update(Collection $c, $id)
	{
		return $this->mysqli->query(
			'update collections set `name` = \'' . $this->escape($c->name) . '\'
			, `has_description` = ' . $this->escape($c->has_description) . '
			, comments_add_email = \'' . $this->escape($c->comments_add_email) . '\'
			, has_image = ' . $this->escape($c->has_image) . ', has_content = ' . $this->escape($c->has_content) . '
			, has_comments = ' . $this->escape($c->has_comments) . '
			, comments_accepted_default = ' . $this->escape($c->comments_accepted_default) . '
			 where id = ' . $id);
	}
	
	/* Deletes the Collection with this id
	 * @id: the ID of the Collection to delete
	 * returns: true on success, false on failure */
	public function Delete($id)
	{
		return (
		$this->mysqli->query('delete cic.* from collectionitems_content as cic, collectionitems as ci where ci.collection_id = ' . $id . ' and ci.id = cic.item_id') &&
		$this->mysqli->query('delete cit.* from collectionitems_tags as cit, collectionitems as ci where ci.collection_id = ' . $id . ' and ci.id = cit.item_id') &&
		$this->mysqli->query('delete cic.* from collectionitems_comments as cic, collectionitems as ci where ci.collection_id = ' . $id . ' and ci.id = cic.item_id') &&
		$this->mysqli->query('delete from collectionitems where collection_id = ' . $id) &&
		$this->mysqli->query('delete from collectiontags where collection_id = ' . $id) &&
		$this->mysqli->query('update pages set collection_id = 0, collection_limit = 0 where collection_id = ' . $id) &&
		$this->mysqli->query('delete from collections where id = ' . $id));
	}
}

define('COMMENT_NOT_ACCEPTED', 0);
define('COMMENT_ACCEPTED', 1);
define('COMMENT_ALL', 2);

class DbCommentAdmin extends DbComment
{

	/* Gets all comments for a specific collectionItem
	 * @item_id: the id of the CollectionItem
	 * @acceptedEnum: 0 = all comments, 1 = accepted comments, 2 = non-accepted comments
	 * returns: an array of Comment objects or an empty array */
	public function GetAll($item_id, $acceptedEnum)
	{
		$accepted = $acceptedEnum == COMMENT_ALL ? 'accepted' : $acceptedEnum;	
		$comments = array();
		if ($result = $this->mysqli->query(
			'select * from collectionitems_comments where item_id = ' . $item_id . '
			and accepted = ' . $accepted . ' order by date_created'))
		{
			while ($obj = $result->fetch_object())
			{
				$comments[] = $this->map($obj);
			}
			$result->close();
		}
		return $comments;
	}
	
	/* Gets comments that are not accepted from a specific collection
	 * @collectionid: the id of the Collection where the comments are from
	 * returns: an array of comments, or an empty array */
	public function GetNonAcceptedFromCollection($collectionid, $specification)
	{
		if (empty($specification))
			$specification = 'specification';
		else
			$specification = "'$specification'";
		$comments = array();
		if ($result = $this->mysqli->query(
			'select cic.* from collectionitems_comments as cic, collectionitems as ci
			where ci.collection_id = ' . $collectionid . ' and ci.id = cic.item_id and cic.accepted = 0
			and cic.specification = ' . $specification . '
			order by date_created'))
		{
			while ($obj = $result->fetch_object())
			{
				$comments[] = $this->map($obj);
			}
			$result->close();
		}
		return $comments;
	}
	
	/* Accepts a comment
	 * @id: the id of the comment
	 * returns: true on success, false on failure */
	public function accept($id)
	{
		return $this->mysqli->query(
			'update collectionitems_comments set accepted = 1, specification = \'none\' where id = ' . $id);
	}

	/* Deletes a comment
	 * @id: the id of the comment
	 * returns: true on success, false on failure */
	public function delete($id)
	{
		return $this->mysqli->query(
			'delete from collectionitems_comments where id = ' . $id);
	}
}

?>
