<?php

class DbMedia extends DbEntity
{
	/* Maps database reader object to Media object
	 * @obj: database reader object
	 * returns: Media object */
	private function map($obj)
	{
		$media = new Media();
		$media->Id = $obj->id;
		$media->Name = $obj->name;
		$media->Type = $obj->media_type;
		$media->Description = $obj->description;
		return $media;
	}

	/* Gets all Media objects, optionally filtered by mediatype or tag
	 * @mediaType: optional type of media: image, document, video, music, other
	 * @tagid: optional id of tag that the Media objects must be tagged with
	 * returns: an array of Media objects, or an empty array */
	public function GetAll($mediaType, $tagid)
	{
		$items = array();
		
		$query = 'select * from media order by media_type, name';
		if (isset($mediaType))
		{
			if (isset($tagid) && $tagid != 0)
			{
				$query = 'select m.* from media as m, media_tags as mt
					where mt.tag_id = ' . $this->escape($tagid) . ' and mt.media_id = m.id
					and m.media_type = \'' . $this->escape($mediaType) . '\'
					group by m.id order by m.media_type, m.name';
			}
			else
			{
				$query = 'select m.* from media as m
					where m.media_type = \'' . $this->escape($mediaType) . '\' order by m.media_type, m.name';
			}
		}
		else
		{
			if (isset($tagid) && $tagid != 0)
			{
				$query = 'select m.* from media as m, media_tags as mt
					where mt.tag_id = ' . $this->escape($tagid) . ' and mt.media_id = m.id
					group by m.id order by m.media_type, m.name';
			}
		}
		
		if ($result = $this->mysqli->query($query))
		{
			while ($obj = $result->fetch_object())
			{
				$items[] = $this->map($obj);
			}
			$result->close();
		}
		
		return $items;
	}
	
	/* Gets a specific Media object by its ID
	 * @id: the ID of the Media to get
	 * returns: a Media object or null */
	public function GetById($id)
	{
		$item = null;
		
		if ($result = $this->mysqli->query(
			'select * from media where id = ' . $this->escape($id)))
		{
			if ($obj = $result->fetch_object())
			{
				$item = $this->map($obj);
			}
			$result->close();
		}
		
		return $item;
	}
	
	/* Adds a mediafile to the database and uploads it into the uploads directory
	 * @file: a $_FILES[] file (from an upload form with input type='file'
	 * @desc: the description of the mediafile
	 * returns: the newly created Media object or null on error */
	public function Add($file, $desc)
	{
		$fileName = $file['name'];
		$mediaType = Media::GetMediaType($fileName);
		$fileNameNew = $this->GetUniqueFileName($file['name'], WE_PATH_ROOT . '/webessence/uploads/' . $mediaType);
		
		if (move_uploaded_file($file['tmp_name'], WE_PATH_ROOT . '/webessence/uploads/' . $mediaType . '/' . $fileNameNew))
		{
			if (chmod(WE_PATH_ROOT . '/webessence/uploads/' . $mediaType . '/' . $fileNameNew, 0644))
			{
				if ($this->mysqli->query(
					'insert into media set name = \'' . $this->escape($fileNameNew) . '\', description = \'' . $this->escape($desc) . '\',
					media_type = \'' . $this->escape($mediaType) . '\''))
				{
					return $this->GetById($this->mysqli->insert_id);
				}
			}
		}
		return null;
	}
	
	/* Returns $fileName or a updated filename so that it is unique
	 * $fileName: a filename (only filename, no directories)
	 * @dir: a directory without trailing slash
	 * returns: $fileName or a changed one if there was already a file with the same name */
	private function GetUniqueFileName($fileName, $dir)
	{
		$pathInfo = pathinfo($fileName);
		
		$counter = 1;
		while (is_file($dir . '/' . $fileName))
		{
			$fileName = $pathInfo['filename'] . $counter . '.' . $pathInfo['extension'];
			$counter++;
		}
		return $fileName;
	}
	
	/* Deletes a Media object with ID $id
	 * @id: the id of the Media object
	 * returns: true on succes, false on failure */
	public function Delete($id)
	{
		$item = $this->GetById($id);
		
		if ($item === null) return false;
		
		if (unlink(WE_PATH_ROOT . '/webessence/uploads/' . $item->Type . '/' . $item->Name))
		{
			$q_id = $this->escape($id);
			if ($this->mysqli->query(
				'delete from media_tags where media_id = ' . $q_id))
			{
				return $this->mysqli->query(
					'delete from media where id = ' . $q_id);
			}
		}
		return false;
	}
	
	/* Updates a Media object ID $id with the values from mediaNew
	 * @id: the id of the Media object to update
	 * @mediaNew: a Media object that is used to update the old Media object
	 * returns: true on success, false on failure */
	public function Update($id, Media $mediaNew)
	{
		return $this->mysqli->query(
			'update media set description = \'' . $this->escape($mediaNew->Description) . '\' where id = ' . $this->escape($id));
	}
}

?>
