<?php

class WeTemplate
{
	public $id;
	public $name;
	public $description;
	public $content; //array of contentid => description
	
	function __construct()
	{
		$this->content = array();
	}
}

class DbWeTemplate extends DbEntity
{
	/* Returns a WeTemplate by its name
	 * @name: the name of the template
	 * returns: a WeTemplate object or null */
	public function GetByName($name)
	{
		$template = null;
		if ($result = $this->mysqli->query(
			'select * from templates where `name` = \'' . $this->escape($name) . '\''))
		{
			if ($obj = $result->fetch_object())
			{
				$template = new WeTemplate();
				$template->name = $name;
				$template->description = $obj->description;
				$template->id = $obj->id;
			}
			$result->close();
			
			if ($result = $this->mysqli->query(
				'select contentid, description from template_contentids where templateid = ' . $template->id))
			{
				while ($obj = $result->fetch_object())
				{
					$template->content[$obj->contentid] = $obj->description;
				}
				$result->close();
			}
		}
		return $template;
	}
	
	/* Gets all templates and contentids of the templates this user is allowed to edit
	 * @userid: ID of the user
	 * returns: an associative array where key = templateid, value is a WeTemplate object
	 * (see GetSurroundingVisible function in /webessence/includes/db_page.php) */
	public function GetTemplatesAndContentIds($userid)
	{
		$list = array();
		$query = null;
		if ($userid == 1)
			$query = 'select t.id, t.`name`, tc.contentid, tc.description from templates as t, template_contentids as tc where t.id = tc.templateid order by t.`name`, tc.contentid';
		else
			$query = 'select t.id, t.`name`, utc.contentid, tc.description from templates as t, user_template_contentids as utc, template_contentids as tc where utc.userid = ' . $userid . ' and utc.templateid = t.id and t.id = tc.templateid and utc.contentid = tc.contentid order by t.`name`, utc.contentid';
		if ($result = $this->mysqli->query($query))
		{
			while ($obj = $result->fetch_object())
			{
				$t = null;
				if (!array_key_exists($obj->id, $list))
				{
					$t = new WeTemplate();
					$t->id = $obj->id;
					$t->name = $obj->name;
					$list[$obj->id] = $t;
				}
				else
				{
					$t = $list[$obj->id];
				}
				$t->content[$obj->contentid] = $obj->description;
			}
			$result->close();
		}
		return $list;
	}
	
	/* Gets all templates this user has permission for (admin all templates, regular user must explicitly been set)
	 * @userid: the id of the user
	 * returns: an array[templateid] => WeTemplate object */
	public function GetAll($userid)
	{
		$templates = array();
		$query = 'select t.id, t.`name`, t.description from templates as t, user_templates as uts where uts.userid = ' . $userid . ' and uts.template_id = t.id order by t.`name`';
		if ($userid == 1)
			$query = 'select id, `name`, description from templates order by `name`';
		if ($result = $this->mysqli->query($query))
		{
			while ($obj = $result->fetch_object())
			{
				$tpl = new WeTemplate();
				$tpl->id = $obj->id;
				$tpl->name = $obj->name;
				$tpl->description = $obj->description;
				$templates[$obj->id] = $tpl;
			}
			$result->close();
		}
		return $templates;
	}
	
	/* Adds a new template
	 * @templateName: name of template
	 * @description: description of the template
	 * returns: the id of the created page or 0 */
	public function Add($templateName, $description)
	{
		if ($this->mysqli->query('insert into templates set `name` = \'' . $templateName . '\', description = \'' . $this->escape($description) . '\''))
			return $this->mysqli->insert_id;
		return 0;
	}
	
	/* Updates the description of a template
	 * @description: the new description
	 * @templateName: the name of the template
	 * returns: true or false */
	public function UpdateDescription($description, $templateName)
	{
		return $this->mysqli->query('update templates set description = \'' . $this->escape($description) . '\' where `name` = \'' . $templateName . '\'');
	}
	
	/* Deletes a template
	 * @templateId: the id of the template
	 * returns: true or false */
	public function Delete($templateId)
	{
		return (
			$this->mysqli->query('delete from user_template_contentids where templateid = ' . $templateId) &&
			$this->mysqli->query('delete from user_templates where template_id = ' . $templateId) &&
			$this->mysqli->query('delete from template_contentids where templateid = ' . $templateId) &&
			$this->mysqli->query('delete from templates where id = ' . $templateId));
	}
	
	/* Updates a contentid of a template
	 * @name: name of the template
	 * @oldContentid: the old contentid
	 * @newContentid: the new contentid
	 * @newDescription: the new description
	 * returns: true or false */
	public function UpdateContentId($name, $oldContentid, $newContentid, $newDescription)
	{
	
		$oldTemplate = $this->GetByName($name);
		if ($oldTemplate == null)
			return false;
		
		if (!$this->mysqli->query('update template_contentids set contentid = \'' . $newContentid . '\', description = \'' . $newDescription . '\' where templateid = ' . $oldTemplate->id . ' and contentid = \'' . $oldContentid . '\''))
			return false;
		
		if ($oldContentid != $newContentid)
			if (!$this->mysqli->query('update pages_content as pc, pages as p set pc.content_id = \'' . $newContentid . '\' where p.templateid = \'' . $oldTemplate->id . '\' and p.id = pc.page_id and pc.content_id = \'' . $oldContentid . '\''))
				return false;
				
		return true;
	}
	
	public function GetPagesByTemplateId($id)
	{
		$pages = array();
		if ($result = $this->mysqli->query(
			'select id from pages where templateid = ' . $id))
		{
			while ($obj = $result->fetch_object())
				$pages[] = $obj->id;
			$result->close();
		}
		return $pages;
	}
	
	/* Deletes a contentid from a template
	 * @templateName: the name of the template
	 * @contentId: the contentid to delete
	 * @returns: true or false */
	public function DeleteContentId($templateName, $contentId)
	{
		$template = $this->GetByName($templateName);
		if ($template == null)
			return false;
		
		return (
			$this->mysqli->query('delete from template_contentids where templateid = ' . $template->id . ' and contentid = \'' . $contentId . '\'') &&
			$this->mysqli->query('delete from user_template_contentids where templateid = ' . $template->id . ' and contentid = \'' . $contentId . '\'') &&
			$this->mysqli->query('delete pc from pages_content as pc, pages as p where p.templateid = ' . $template->id . ' and p.id = pc.page_id and pc.content_id = \'' . $contentId . '\' and pc.content = \'\'') &&
			$this->mysqli->query('update pages_content as pc, pages as p set pc.content_id = \'\' where p.templateid = ' . $template->id . ' and p.id = pc.page_id and pc.content_id = \'' . $contentId . '\' and pc.content <> \'\''));
		
	}
	
	/* Adds a new contentid to a template
	 * @templateName: the name of the template
	 * @contentId: the new contentid
	 * @description: the description of the contentid
	 * returns: true or false */
	public function AddContentId($templateName, $contentId, $description)
	{
		$template = $this->GetByName($templateName);
		if ($template == null)
			return false;
		if (!$this->mysqli->query('insert into template_contentids set templateid = ' . $template->id . ', contentid = \'' . $contentId . '\', description = \'' . $this->escape($description) . '\''))
			return false;
		
		$pages = $this->GetPagesByTemplateId($template->id);
		foreach ($pages as $pageId)
		{
			if (!$this->mysqli->query('insert into pages_content set page_id = ' . $pageId . ', content_id = \'' . $contentId . '\', content = \'\''))
				return false;
		}
		
		return true;
	}
}

class DbPageAdmin extends DbPage
{

	/* Gets all pages
	 * returns: an associative array
	 * (see GetSurroundingVisible function in /webessence/includes/db_page.php) */
	public function GetAllFull()
	{
		// false means: visible AND non visible pages (true means only visible pages)
		return $this->GetAll(false);
	}
	
	/* Inserts a new Page
	 * @page: the Page to create
	 * returns: the id of the newly created Page or 0 on failure */
	public function Create(Page $page, WeTemplate $template)
	{
		$isUnique = false;
		
		if ($result = $this->mysqli->query('select count(*) from pages where prepath = \'' . $this->escape($page->prepath) . '\' and pathname = \'' . $this->escape($page->pathname) . '\' limit 1'))
		{
			if ($row = $result->fetch_row())
			{
				if ($row[0] == 0)
					$isUnique = true;
			}
			$result->close();
		}
		
		if (!$isUnique)
			return 0;
	
		$sortOrder = 0;
		
		if ($result = $this->mysqli->query(
			'select max(sortorder) from pages where parentid = ' . $this->escape($page->parentid)))
		{
			if ($row = $result->fetch_row())
			{
				$sortOrder = $row[0] + 1;
			}
			$result->close();
		}

		if ($page->is_homepage)
			if (!$this->mysqli->query('update pages set is_homepage = 0 where is_homepage = 1'))
				return false;

		if (!$this->mysqli->query(
			'insert into pages set menuname = \'' . $this->escape($page->menuname) . '\'
			, title = \'' . $this->escape($page->title) . '\', pathname = \'' . $this->escape($page->pathname) . '\',
			parentid = ' . $this->escape($page->parentid) . ', sortorder = ' . $sortOrder . ',
			depth = ' . $this->escape($page->depth) . ', prepath = \'' . $this->escape($page->prepath) . '\', 
			visible = ' . $this->escape($page->visible) . ', templateid = ' . $page->templateid . ',
			locked = ' . $this->escape($page->locked) . ',
			collection_id = ' . $this->escape($page->collection_id) . ', collection_limit = ' . $this->escape($page->collection_limit) . ',
			cache_minutes = ' . $page->cache_minutes . ', is_homepage = ' . $page->is_homepage .
			', meta_keywords = \'' . $this->escape($page->meta_keywords) . '\', meta_description = \'' . $this->escape($page->meta_description) . '\''))
		{
			return 0;
		}
			
		$id = $this->mysqli->insert_id;
		
		foreach ($template->content as $contentId => $useless)
		{
			if (!$this->mysqli->query('insert into pages_content set page_id = ' . $id . ', content_id = \'' . $contentId . '\', content = \'\''))
				return 0;
		}
		
		return $id;
	}
	
	/* Returns content that doesn't belong to a contentid from this page
	 * @pageId: is of the page
	 * returns: array[id] => content */
	public function GetOrphanedContent($pageId)
	{
		$list = array();
		if ($result = $this->mysqli->query('select * from pages_content where page_id = ' . $pageId . ' and content_id = \'\''))
		{
			while ($obj = $result->fetch_object())
				$list[$obj->id] = $obj->content;
			$result->close();
		}
		return $list;
	}
	
	/* Deletes content
	 * @id: id of the pages_content to delete
	 * returns: true or false */
	public function DeleteOrphanedContent($id)
	{
		return $this->mysqli->query('delete from pages_content where id = ' . $id);
	}
	
	/* Promotes orphaned content to a contentid
	 * @pageid: id of page
	 * @id: id of pages_content (the orphaned content)
	 * @contentid: the contentid to promote the content to
	 * returns: true or false */
	public function set_orphaned_content($pageid, $id, $contentid)
	{
		return (
			$this->mysqli->query('delete from pages_content where page_id = ' . $pageid . ' and content_id = \'' . $contentid . '\'') &&
			$this->mysqli->query('update pages_content set content_id = \'' . $contentid . '\' where id = ' . $id));
	}
	
	/* Updates the collection properties of a page
	 * @id: id of page
	 * @collectionid: the id of the collection
	 * @collectionlimit: number of items to show on each page
	 * returns: true or false */
	public function update_collection($id, $collectionid, $collectionlimit)
	{
		return $this->mysqli->query('update pages set collection_id = ' . $collectionid .
			', collection_limit = ' . $collectionlimit . ' where id = ' . $id);
	}
	
	/* Sets the cache for this page
	 * @id: id of page
	 * @cache: cache in minutes or 0 for no caching
	 * returns: true or false */
	public function set_cache($id, $cache)
	{
		return $this->mysqli->query('update pages set cache_minutes = ' . $cache . ' where id = ' . $id);
	}
	
	/* Updates the used template for this page
	 * @id: id of page
	 * @templateNew: WeTemplate object of new template
	 * @templateOld: WeTemplate object of old template
	 * returns: true or false */
	public function update_template($id, $templateNew, $templateOld)
	{	
		// Make all contentids orphaned (empty) that don't belong to the new template and have content
		foreach ($templateOld->content as $contentId => $useless)
		{
			if (!array_key_exists($contentId, $templateNew->content))
				if (!$this->mysqli->query('update pages_content set content_id = \'\' where page_id = ' . $id . ' and content_id = \'' . $contentId . '\' and content <> \'\''))
					return false;
		}
		
		// Add all contentids that dont't belong to the old template
		foreach ($templateNew->content as $contentId => $useless)
		{
			if (!array_key_exists($contentId, $templateOld->content))
				if (!$this->mysqli->query('insert into pages_content set page_id = ' . $id . ', content_id = \'' . $contentId . '\', content = \'\''))
					return false;
		}
		
		return $this->mysqli->query('update pages set templateid = \'' . $this->escape($templateNew->id) . '\' where id = ' . $id);
	}
	
	/* Makes this page the homepage
	 * @id: id of the page to set as homepage
	 * returns: true or false */
	public function set_as_homepage($id)
	{
		return (
			$this->mysqli->query('update pages set is_homepage = 0 where is_homepage = 1') &&
			$this->mysqli->query('update pages set is_homepage = 1 where id = ' . $id));
	}
	
	/* Updates lock of this page
	 * @id: id of page
	 * @lock: 0 or 1
	 * returns: true or false */
	public function set_lock($id, $lock)
	{
		return $this->mysqli->query('update pages set locked = ' . $lock . ' where id = ' . $id);
	}
	
	/* Updates basic properties of a page
	 * @id: id of page
	 * @menuname: new name of menu
	 * @title: new title
	 * @pathname: new pathname
	 * @visible: 0 or 1
	 * returns: true or false */
	public function update_basic($id, $menuname, $title, $visible)
	{
		$oldpage = $this->GetById($id);
		
		if ($oldpage->locked)
			return false;
		
		return $this->mysqli->query('update pages set menuname = \'' . $this->escape($menuname) . '\'' .
			', title = \'' . $this->escape($title) . '\'' .
			', visible = ' . $visible . ' where id = ' . $id);
	}
	
	public function update_seo($id, $keywords, $description, $pathname)
	{
		$oldpage = $this->GetById($id);
		
		if ($oldpage->locked)
			return false;
			
		// If the pathname is changed, the prepath of all children must change!
		if ($oldpage->pathname != $pathname)
		{
			if (!$this->mysqli->query(
				'update pages set prepath = \'' . $this->escape($oldpage->prepath . $pathname) . '/\'' .
				' where prepath like \'' . $this->escape($oldpage->prepath . $oldpage->pathname) . '/%\''))
				return false;
				
			if (!$this->mysqli->query('update pages set pathname = \'' . $this->escape($pathname) . '\' where id = ' . $id))
				return false;
			
			// Also clear the cache for this page
			$this->clear_cache_page($oldpage);
		}
		
		return $this->mysqli->query('update pages set meta_keywords = \'' . $this->escape($keywords) . '\'' .
			', meta_description = \'' . $this->escape($description) . '\' where id = ' . $id);
	}
	
	/* Checks if this page has a cache file
	 * @page: Page object
	 * returns: true or false */
	public function has_cache_dir(Page $page)
	{
		return is_dir('../cache/' . $page->prepath . $page->pathname);
	}
	
	/* Checks if this website has at least 1 cache file
	 * returns: true or false */
	public function has_cache_all()
	{
		if ($files = scandir('../cache'))
		{
			foreach ($files as $file)
			{
				if ($file != '.' && $file != '..')
					return true;
			}
		}
		
		return false;
	}
	
	/* Clears the cache files for this page
	 * @page: the Page object
	 * returns: void */
	private function clear_cache_page(Page $page)
	{
		if ($page != null && is_dir('../cache/' . $page->prepath . $page->pathname))
			$this->rm_recursive('../cache/' . $page->prepath . $page->pathname, true);
		if ($page->is_homepage)
		{
			// Haal ook alle files weg uit cache zelf
			if ($files = scandir('../cache/'))
			{
				foreach ($files as $file)
					if (!is_dir('../cache/' . $file))
						unlink('../cache/' . $file);
			}
		}
	}
	
	/* Clears the cache for this page
	 * @id: id of page
	 * returns: void */
	public function clear_cache($id)
	{
		$page = $this->GetById($id);
		$this->clear_cache_page($page);
	}
	
	/* Clears cache of whole website
	 * returns: void */
	public function clear_cache_all()
	{
		$this->rm_recursive('../cache', false);
	}
	
	/* Deletes a directory and ALL files and directories in it.
	 * @dir: the directory to delete
	 * returns: true or false */
	private function rm_recursive($dir, $includeDir)
	{
		if ($files = scandir($dir))
		{
			foreach ($files as $file)
			{
				if ($file == '.' | $file == '..')
					continue;
				
				if (is_dir($dir . '/' . $file))
					$this->rm_recursive($dir . '/' . $file, true);
				else
					unlink($dir . '/' . $file);
			}
			if ($includeDir)
				rmdir($dir);
		}
		
		return true;
	}
	
	/* Updates the content of a specific Page
	 * @id: the id of the Page to update
	 * @content: the new content
	 * returns: true on succes, false on failure */
	public function UpdateContent($id, $contentId, $content)
	{
		return $this->mysqli->query('update pages_content set content = \'' . $this->escape($content) . '\'
			where page_id = ' . $this->escape($id) . ' and content_id = \'' . $contentId . '\'');
	}
	
	/* Deletes a specific Page
	 * @id: the id of the Page to delete
	 * returns: true on success, false on failure */
	public function Delete($id)
	{
		$page = $this->GetById($id);
		if ($page === null) return false;
		
		return (
			// Delete all children (subpages)
			$this->mysqli->query('delete pc.* from pages as p, pages_content as pc
				where p.prepath like \'' . $page->prepath . $page->pathname . '/%\' and p.id = pc.page_id') &&
			$this->mysqli->query('delete from pages where prepath like \'' . $page->prepath . $page->pathname . '/%\'') &&
			// Delete this page
			$this->mysqli->query('delete from pages_content where page_id = ' . $id) &&
			$this->mysqli->query('delete from pages where id = ' . $id) &&
			// Update sort order
			$this->mysqli->query(
				'update pages set sortorder = sortorder - 1
				where parentid = ' . $page->parentid . ' and sortorder > ' . $page->sortorder));
	}
	
	/* Updates the sort order of a Page (one up or one down)
	 * @id: the id of the Page
	 * @dir: the direction of the sort order update: 'up' or 'down'
	 * returns: true on success, false on failure */
	public function Sort($id, $dir)
	{
		$page = $this->GetById($id);
		if ($page === null) return false;
		$one = $dir == 'up' ? -1 : 1;
		
		if (!$this->mysqli->query(
			'update pages set sortorder = sortorder - ' . $one . '
			where parentid = ' . $this->escape($page->parentid) . ' and sortorder = ' . $this->escape($page->sortorder) . ' + ' . $one))
		{
			return false;
		}

		if ($this->mysqli->affected_rows == 1)
		{
			return $this->mysqli->query(
				'update pages set sortorder = sortorder + ' . $one . '
				where id = ' . $this->escape($id));
		}
		return true;
	}
	
	/* Gets all root pages (with parentid 0 or -1)
	 * returns: an array with Page objects or an empty array */
	public function GetRootPages()
	{
		$pages = array();
		
		if ($result = $this->mysqli->query(
			'select * from pages where parentid <= 0 order by parentid, sortorder'))
		{
			while ($obj = $result->fetch_object())
			{
				$pages[] = $this->map($obj);
			}
			$result->close();
		}
		return $pages;
	}
	
	/* Gets all templates the user can choose from
	 * returns: an array of templates or an empty array */
	//~ public function GetTemplates()
	//~ {
		//~ $files = array();
		//~ if ($d = opendir('../templates'))
		//~ {
			//~ while (($file = readdir($d)) !== false)
				//~ if ($file != '.' && $file != '..') $files[] = $file;
			//~ closedir($d);
		//~ }
		//~ return $files;
	//~ }
}

?>
