<?php

class DbTagAdmin extends DbTag
{

	/* Gets tags from a collectionitem
	 * @itemid: the id of the collectionitem
	 * returns: an assoccitaive array, where key = tagid, value = tagname or an empty array */
	public function GetTagsFromCollectionItem($itemid)
	{
		$tags = array();
		if ($result = $this->mysqli->query(
			'select collectiontags.* from collectiontags, collectionitems_tags as cit
			where cit.item_id = ' . $this->escape($itemid) . '
			and cit.tag_id = collectiontags.id order by collectiontags.tagname'))
		{
			while ($obj = $result->fetch_object())
			{
				$tags[$obj->id] = $obj->tagname;
			}
			$result->close();
		}
		return $tags;
	}

	/* Gets all mediatags
	 * returns: an assoccitaive array, where key = tagid, value = tagname or an empty array */
	public function GetAllMedia()
	{
		$tags = array();
		if ($result = $this->mysqli->query('select * from mediatags order by tagname'))
		{
			while ($obj = $result->fetch_object())
			{
				$tags[$obj->id] = $obj->tagname;
			}
			$result->close();
		}
		return $tags;
	}
	
	/* Gets all tags from a specific Media item
	 * @itemid: the id of the Media item
	 * returns: an assoccitaive array, where key = tagid, value = tagname or an empty array */
	public function GetTagsFromMediaItem($itemid)
	{
		$tags = array();
		if ($result = $this->mysqli->query(
			'select mediatags.* from mediatags, media_tags as mti
			where mti.media_id = ' . $this->escape($itemid) . '
			and mti.tag_id = mediatags.id order by mediatags.tagname'))
		{
			while ($obj = $result->fetch_object())
			{
				$tags[$obj->id] = $obj->tagname;
			}
			$result->close();
		}
		return $tags;
	}
	
	/* Adds a mediatag
	 * @name: the name of the tag
	 * returns: true on success, false on failure */
	public function AddMedia($name)
	{
		return $this->mysqli->query('insert into mediatags set tagname = \'' . $this->escape($name) . '\'');
	}
	
	/* Adds a collectiontag
	 * @name: the name of the tag
	 * @collectionid: the id of the collection
	 * returns: true on success, false on failure */
	public function AddCollection($name, $collectionid)
	{
		return $this->mysqli->query('insert into collectiontags set collection_id = ' . $collectionid . ', tagname = \'' . $this->escape($name) . '\'');
	}
	
	/* Deletes a mediatag
	 * @id: the id of the tag
	 * returns: true on success, false on failure */
	public function DeleteMedia($id)
	{
		return (
			$this->mysqli->query('delete from media_tags where tag_id = ' . $this->escape($id)) &&
			$this->mysqli->query('delete from mediatags where id = ' . $this->escape($id)));
	}
	
	/* Deletes a collectiontag
	 * @id: the id of the tag
	 * returns: true on success, false on failure */
	public function DeleteCollection($id)
	{
		return (
			$this->mysqli->query('delete from collectionitems_tags where tag_id = ' . $this->escape($id)) &&
			$this->mysqli->query('delete from collectiontags where id = ' . $this->escape($id)));
	}
	
	/* Updates a mediaitem with tags
	 * @itemid: the id of the media item
	 * @tagids: an array consists of tagids - this is the new situation
	 * returns: true on success, false on failure */
	public function UpdateMediaItemWithTags($itemId, $tagids)
	{
		if (empty($tagids)) return $this->mysqli->query('delete from media_tags where media_id = ' . $this->escape($itemId));
		
		$oldTags = $this->GetTagsFromMediaItem($itemId);
		
		foreach ($oldTags as $oldTagId => $oldTag)
		{
			if (!in_array($oldTagId, $tagids))
			{
				if (!$this->mysqli->query('delete from media_tags where media_id = ' . $this->escape($itemId) . ' and tag_id = ' . $this->escape($oldTagId)))
					return FALSE;
			}
		}
		
		foreach ($tagids as $tagid)
		{
			if (!isset($oldTags[$tagid]))
			{
				if (!$this->mysqli->query('insert into media_tags set media_id = ' . $this->escape($itemId) . ', tag_id = ' . $this->escape($tagid)))
					return false;
			}
		}
		
		return TRUE;
	}
	
	/* Updates a collectionitem with tags
	 * @itemid: the id of the collection item
	 * @tagids: an array consists of tagids - this is the new situation
	 * returns: true on success, false on failure */
	public function UpdateCollectionItemWithTags($itemId, $tagids)
	{
		if (empty($tagids)) return $this->mysqli->query('delete from collectionitems_tags where item_id = ' . $this->escape($itemId));
		
		$oldTags = $this->GetTagsFromCollectionItem($itemId);
	
		foreach ($oldTags as $oldTagId => $oldTag)
		{
			if (!in_array($oldTagId, $tagids))
			{
				if (!$this->mysqli->query('delete from collectionitems_tags where item_id = ' . $this->escape($itemId) . ' and tag_id = ' . $this->escape($oldTagId)))
					return FALSE;
			}
		}
		
		foreach ($tagids as $tagid)
		{
			if (!isset($oldTags[$tagid]))
			{
				if (!$this->mysqli->query('insert into collectionitems_tags set item_id = ' . $this->escape($itemId) . ', tag_id = ' . $this->escape($tagid)))
					return false;
			}
		}
		
		return TRUE;
	}
}

?>