<?php

class DbUser extends DbEntity
{
	/* Maps a database reader object to a User
	 * @obj: database reader
	 * returns: User */
	private function map($obj)
	{
		$user = new User();
		$user->id = $obj->id;
		$user->name = $obj->name;
		$user->password = $obj->pwd;
		$user->email = $obj->email;
		$user->description = $obj->description;
		$user->manage_media = $obj->manage_media;
		$user->sort_pages = $obj->sort_pages;
		$user->manage_collections = $obj->manage_collections;
		$user->manage_users = $obj->manage_users;
		$user->manage_comments = $obj->manage_comments;
		$user->manage_collectionitems = $obj->manage_collectionitems;
		$user->set_cache = $obj->set_cache;
		$user->manage_config = $obj->manage_config;
		$user->manage_page_collections = $obj->manage_page_collections;
		$user->manage_pages = $obj->manage_pages;
		$user->set_lock = $obj->set_lock;
		$user->manage_templates = $obj->manage_templates;
		return $user;
	}
	
	/* Gets a user by its id
	 * @id: id of User
	 * returns: User or null */
	public function GetById($id)
	{
		$user = null;
		if ($result = $this->mysqli->query(
			'select * from users where id = ' . $this->escape($id)))
		{
			if ($obj = $result->fetch_object())
			{
				$user = $this->map($obj);
			}
			$result->close();
			
			if ($user != null)
			{
				$dbTemplate = new DbWeTemplate($this->mysqli);
				$user->templates = $dbTemplate->GetAll($id);
				$user->template_contentids = $dbTemplate->GetTemplatesAndContentIds($id);
			}
		}
		return $user;
	}
	
	/* Gets a user by its name
	 * @name: name of user
	 * returns: User or null */
	public function GetByName($name)
	{
		$user = null;
		if ($result = $this->mysqli->query(
			'select * from users where name = \'' . $this->escape($name) . '\''))
		{
			if ($obj = $result->fetch_object())
			{
				$user = $this->map($obj);
			}
			$result->close();
			
			if ($user != null)
			{
				$dbTemplate = new DbWeTemplate($this->mysqli);
				$user->templates = $dbTemplate->GetAll($user->id);
				$user->template_contentids = $dbTemplate->GetTemplatesAndContentIds($user->id);
			}
		}
		return $user;
	}
	
	/* Gets all users
	 * returns: array of Users or an empty array */
	public function GetAll()
	{
		$users = array();
		if ($result = $this->mysqli->query(
			'select * from users order by name'))
		{
			while ($obj = $result->fetch_object())
			{
				$users[] = $this->map($obj);
			}
			$result->close();
		}
		return $users;
	}
	
	/* Inserts a new user in the database
	 * @user: the new User
	 * returns: true on succes, false on failure */
	public function Create($user)
	{
		if (!$this->mysqli->query(
			'insert into users set name = \'' . $this->escape($user->name) . '\',
			pwd = \'' . $this->escape($user->password) . '\',
			email = \'' . $this->escape($user->email) . '\', description = \'' . $this->escape($user->description) . '\', manage_media = ' . $user->manage_media . ',
			sort_pages = ' . $user->sort_pages . ',
			manage_collections = ' . $user->manage_collections . ', manage_users = ' . $user->manage_users . ',
			manage_comments = ' . $user->manage_comments . ',
			manage_collectionitems = ' . $user->manage_collectionitems . ',
			set_cache = ' . $user->set_cache . ', manage_config = ' . $user->manage_config . ',
			manage_page_collections = ' . $user->manage_page_collections . ', manage_pages = ' . $user->manage_pages . ', set_lock = ' . $user->set_lock .
			', manage_templates = ' . $user->manage_templates))
			return false;
			
		$userid = $this->mysqli->insert_id;
		
		foreach ($user->templates as $templateid)
			if (!$this->mysqli->query('insert into user_templates set userid = ' . $userid . ', template_id = ' . $templateid))
				return false;
			
		foreach ($user->template_contentids as $templateid => $contentids)
		{
			foreach ($contentids as $contentid)
			{
				if (!$this->mysqli->query('insert into user_template_contentids set userid = ' . $userid . ', templateid = ' . $templateid . ', contentid = \'' . $contentid . '\''))
					return false;
			}
		}
		
		return true;
	}
	
	/* Delete the user with this id
	 * @id: the id of the user to delete
	 * returns: true on success, false on failure */
	public function Delete($id)
	{
		if ($id == 1)
			return false;
		$this->mysqli->query('delete from user_templates where userid = ' . $id);
		$this->mysqli->query('delete from user_template_contentids where userid = ' . $id);
		$this->mysqli->query('delete from users where id = ' . $id);
		return true;
	}
	
	/* Update basic properties of this user
	 * @userid: id of the user
	 * @username: new username
	 * @pwdc: new encrypted password (sha1)
	 * @email: new email
	 * @description: new description
	 * returns: true or false */
	public function update_basic($userid, $username, $pwdc, $email, $description)
	{
		$pwdc = empty($pwdc) ? 'pwd' : "'$pwdc'";
		return $this->mysqli->query('update users set `name` = \'' . $this->escape($username) . '\', pwd = ' . $pwdc .
			', email = \'' . $this->escape($email) . '\', description = \'' . $this->escape($description) . '\'' .
			' where id = ' . $userid);
	}
	
	/* Updates the permissions of this user
	 * @userid: id of user
	 * @user: a User object with the new permissions set
	 * returns: true or false */
	public function update_permissions($userid, $user)
	{
		return $this->mysqli->query('update users set manage_media = ' . $user->manage_media .
			', manage_pages = ' . $user->manage_pages . ', sort_pages = ' . $user->sort_pages .
			', manage_collections = ' . $user->manage_collections . ', manage_users = ' . $user->manage_users .
			', manage_comments = ' . $user->manage_comments . ', manage_collectionitems = ' . $user->manage_collectionitems .
			', set_cache = ' . $user->set_cache . ', manage_config = ' . $user->manage_config .
			', manage_page_collections = ' . $user->manage_page_collections .
			', set_lock = ' . $user->set_lock . ', manage_templates = ' . $user->manage_templates .
			' where id = ' . $userid);
	}
	
	/* Updates the templates a user can choose from when creating a new page or updating a page
	 * and the template_contentids (the contentids of the templates this user can edit)
	 * @id: the id of the user
	 * @templates: an array with templateids this user can choose
	 * @template_contentids: array[templateid] => array(contentid1, contentid2)
	 * returns: true or false */
	public function update_template_permissions($id, $templates, $template_contentids)
	{
		$dbTemplate = new DbWeTemplate($this->mysqli);
		$oldTemplates = $dbTemplate->GetAll($id);
		
		// doorloop de oude template en als het id niet voorkomt in de niewue, dan deleteeb
		foreach ($oldTemplates as $templateid => $template)
		{
			if (!in_array($templateid, $templates))
			{
				if (!$this->mysqli->query('delete from user_templates where userid = ' . $id . ' and template_id = ' . $templateid))
					return false;
			}
		}
		
		// doorloop nu de nieuwe templates en als ze niet in de oude voorkomen dan inserten
		foreach ($templates as $templateid)
		{
			if (!array_key_exists($templateid, $oldTemplates))
			{
				if (!$this->mysqli->query('insert into user_templates set userid = ' . $id . ', template_id = ' . $templateid))
				{
					return false;
				}
			}
		}
		
		$oldTemplateContentids = $dbTemplate->GetTemplatesAndContentIds($id);
		
		// Doorloop de oude template_contentids en als het niet voorkomt in de niewue, dan verwijderen uit de tabel voor deze user
		foreach ($oldTemplateContentids as $oldTemplateid => $oldTemplate)
		{
			// Als templateid niet voorkomt in de niewue kun je alles meteen verwijdern van deze template
			if (!array_key_exists($oldTemplateid, $template_contentids))
			{
				$this->mysqli->query('delete from user_template_contentids where userid = ' . $id . ' and templateid = ' . $oldTemplateid);
				continue;
			}
			
			$newContentidsInOldTemplate = $template_contentids[$oldTemplateid];
			
			foreach ($oldTemplate->content as $oldContentid => $oldDescription)
			{
				if (!in_array($oldContentid, $newContentidsInOldTemplate))
				{
					$this->mysqli->query('delete from user_template_contentids where userid = ' . $id . ' and templateid = ' . $oldTemplateid . ' and contentid = \'' . $oldContentid . '\'');
				}
			}
		}
		
		//doorloop de niewue template_contentids en als deze niet voorkoemn in de oude, dan toevoegen aande teale
		foreach ($template_contentids as $templateid => $contentids)
		{
			foreach ($contentids as $contentid)
			{
				if (!array_key_exists($templateid, $oldTemplateContentids) || !array_key_exists($contentid, $oldTemplateContentids[$templateid]->content))
				{
					$this->mysqli->query('insert into user_template_contentids set userid = ' . $id . ', templateid = ' . $templateid . ', contentid = \'' . $contentid . '\'');
				}
			}
		}
		
		return true;
	}
}

?>