var htmlEditor = function(doc, cssClassElements, imgClassElements, lang, editable)
{
	if (lang) this.lang = lang;
	else
	{
		this.lang = {
			"btn_toggleWYSIWYG" : "HTML / design mode",
			"btn_toggleFullscreen" : "Full  / standard screen",
			"btn_executeBold" : "Bold",
			"btn_executeItalic" : "Italic",
			"btn_executeRemoveFormat" : "Remove style",
			"btn_prepareLink" : "Link",
			"btn_prepareImage" : "Image",
			"btn_prepareDocument" : "Other media",
			"btn_prepareTable" : "Table",
			"btn_executeListBullets" : "Bullet list",
			"btn_executeListNumbers" : "Numbered list",
			"btn_showSpecialCharsWindow" : "Insert special characters",
			"sel_block_empty" : "No block...",
			"sel_block_p" : "Paragraph",
			"sel_block_h1" : "Header 1",
			"sel_block_h2" : "Header 2",
			"sel_block_h3" : "Header 3",
			"sel_block_h4" : "Header 4",
			"sel_block_pre" : "Preformatted",
			"sel_class_empty" : "No class...",
			"status_pos" : "Position",
			"cmenu_editLink" : "Edit link",
			"cmenu_editImage" : "Edit image",
			"cmenu_deleteTable" : "Delete table",
			"cmenu_addCaption" : "Add caption",
			"cmenu_deleteCaption" : "Delete caption",
			"cmenu_addThead" : "Add table head",
			"cmenu_deleteThead" : "Delete table head",
			"cmenu_addTfoot" : "Add table foot",
			"cmenu_deleteTfoot" : "Delete table foot",
			"cmenu_deleteColumn" : "Delete column",
			"cmenu_addColumnAfter" : "Add column after current column",
			"cmenu_addColumnBefore" : "Add column before current column",
			"cmenu_deleteRow" : "Delete row",
			"cmenu_addRowAfter" : "Add row after current row",
			"cmenu_addRowBefore" : "Add row before current row",
			"confirm_deleteTable" : "Delete this table?",
			"prompt_caption" : "Caption",
			"popup_btnOk" : "OK",
			"popup_btnCancel" : "Cancel",
			"popup_tableRowCount" : "Rows",
			"popup_tableColCount" : "Columns",
			"popup_tableLegendBasic" : "Basic",
			"popup_tableAlertRowCountNotValid" : "Number of rows is not valid!",
			"popup_tableAlertColCountNotValid" : "Number of columns is not valid",
			"popup_linkHref" : "Target (URL or email)",
			"popup_linkTitle" : "Title",
			"popup_linkAlertHrefNotValid" : "The target is not valid!",
			"popup_linkLegendBasic" : "Basic",
			"popup_imgSrc" : "Source",
			"popup_imgTitle" : "Title",
			"popup_imgFloatNone" : "Standard",
			"popup_imgFloatLeft" : "Left",
			"popup_imgFloatRight" : "Right",
			"popup_imgSelClassEmpty" : "No CSS class...",
			"popup_imgLegendClass" : "CSS class",
			"popup_imgLegendFloat" : "Floating",
			"popup_imgLegendBasic" : "Basic"
		};
	}
	
	var me = this;
	this.doc = doc;

	var editor = doc.createElement("div");
	this.editor = editor;
	editor.className = "editor";
	this.wysiwygMode = true;
	this.isFullscreen = false;
	this.contextMenu = null;
	this.imgClassElements = imgClassElements;
	
	var toolbar = doc.createElement("div");
	this.toolbar = toolbar;
	toolbar.className = "toolbar";
	
	var buttons = [
		{
			"txt" : this.lang.btn_toggleWYSIWYG,
			"func" : "toggleWYSIWYG",
			"img" : "img/html.png"
		},
		{
			"txt" : this.lang.btn_toggleFullscreen,
			"func" : "toggleFullscreen",
			"img" : "img/application.png"
		},
		{
			"txt" : this.lang.btn_executeBold,
			"func" : "executeBold",
			"img" : "img/text_bold.png",
			"tag" : "strong"
		},
		{
			"txt" : this.lang.btn_executeItalic,
			"func" : "executeItalic",
			"img" : "img/text_italic.png",
			"tag" : "em"
		},
		{
			"txt" : this.lang.btn_executeRemoveFormat,
			"func" : "executeRemoveFormat",
			"img" : "img/style_delete.png"
		},
		{
			"txt" : this.lang.btn_prepareLink,
			"func" : "prepareLink",
			"img" : "img/link.png",
			"tag" : "a"
		},
		{
			"txt" : this.lang.btn_prepareImage,
			"func" : "prepareImage",
			"img" : "img/photo.png"
		},
		{
			"txt" : this.lang.btn_prepareDocument,
			"func" : "prepareDocument",
			"img" : "img/page_link.png"
		},
		{
			"txt" : this.lang.btn_prepareTable,
			"func" : "prepareTable",
			"img" : "img/table.png"
		},
		{
			"txt" : this.lang.btn_executeListBullets,
			"func" : "executeListBullets",
			"img" : "img/text_list_bullets.png",
			"tag" : "ul"
		},
		{
			"txt" : this.lang.btn_executeListNumbers,
			"func" : "executeListNumbers",
			"img" : "img/text_list_numbers.png",
			"tag" : "ol"
		},
		{
			"txt" : this.lang.btn_showSpecialCharsWindow,
			"func" : "showSpecialCharsWindow",
			"img" : "img/text_letter_omega.png"
		}
	];
	
	for (var i = 0; i < buttons.length; i++)
	{
		var b = buttons[i];
		var a = doc.createElement("a");
		a.tag = b.tag;
		a.setAttribute("href", "javascript:void(0)");
		a.setAttribute("title", b.txt);
		var img = doc.createElement("img");
		img.setAttribute("src", b.img);
		a.appendChild(img);
		a.onclick = this.callbackCreator(b.func, a);
		toolbar.appendChild(a);
		
		if (i == 1)
		{
			var select = doc.createElement("select");
			select.onchange = this.callbackCreator("executeSelect", select);
			select.onclick = function(e) {e ? e.stopPropagation() : window.event.cancelBubble = true;};
			var keyValues = {
				"" : this.lang.sel_block_empty,
				"p" : this.lang.sel_block_p,
				"h1" : this.lang.sel_block_h1,
				"h2" : this.lang.sel_block_h2,
				"h3" : this.lang.sel_block_h3,
				"h4" : this.lang.sel_block_h4,
				"pre" : this.lang.sel_block_pre
			};
			for (var key in keyValues)
			{
				var option = doc.createElement("option");
				option.$tagName = key;
				option.setAttribute("value", key);
				option.appendChild(doc.createTextNode(keyValues[key]));
				select.appendChild(option);
			}
			toolbar.appendChild(select);
			
			if (cssClassElements)
			{
				select = doc.createElement("select");
				select.onchange = this.callbackCreator("executeSelect", select);
				//select.onclick = function(e) {e ? e.stopPropagation() : window.event.cancelBubble = true;};
				
				var option = doc.createElement("option");
				option.setAttribute("value", "");
				option.appendChild(doc.createTextNode(this.lang.sel_class_empty));
				select.appendChild(option);
				
				for (var j = 0; j < cssClassElements.length; j++)
				{
					var cssClassElement = cssClassElements[j];
					option = doc.createElement("option");
					option.$tagName = cssClassElement.tagName;
					option.$attributeName = "class";
					option.setAttribute("value", cssClassElement.attributeValue != null ? cssClassElement.attributeValue : cssClassElement.tagName);
					option.appendChild(doc.createTextNode(cssClassElement.friendlyText));
					select.appendChild(option);
				}
				
				toolbar.appendChild(select);
			}
		}
	}
	
	editor.appendChild(toolbar);
	
	if (!editable)
	{
		editable = doc.createElement("div");
		editor.appendChild(editable);
	}
	this.editable = editable;
	editable.className = editable.className + " editable";
	editable.setAttribute("contentEditable", "true");
	
	editable.onkeyup = this.callbackCreator("onKeyUp", null);
	editable.onmouseup = this.callbackCreator("onMouseUp", null);
	editable.onfocus = this.callbackCreator("onFocus", null);
	editable.oncontextmenu = function(e)
	{
		me.onContextMenu(e);
	};
	
	var statusbar = doc.createElement("div");
	statusbar.className = "statusbar";
	this.statusbar = statusbar;
	var span = doc.createElement("span");
	span.appendChild(doc.createTextNode(this.lang.status_pos + ": "));
	statusbar.appendChild(span);
	if (htmlEditor.userAgent.indexOf("safari") == -1) statusbar.onclick = this.callbackCreator("statusbarOnClick", null);
	
	editor.appendChild(statusbar);
};

htmlEditor.prototype.addToolbarSeparator = function()
{
	var div = document.createElement("div");
	div.className = "toolbarSep";
	div.appendChild(document.createElement("br"));
	this.toolbar.appendChild(div);
};

htmlEditor.prototype.callbackCreator = function(func, arg)
{
	var me = this;
	return function()
	{
		me[func](arg);
	};
};

htmlEditor.prototype.statusbarOnClick = function(arg)
{
	if (!htmlEditor.is_ie)
		this.editable.focus();
};

htmlEditor.prototype.onContextMenu = function(e)
{
	var currentElement = null;
	if (e)
	{
		e.preventDefault();
		currentElement = e.target;
	}
	else
	{
		e = window.event;
		e.returnValue = false;
		currentElement = e.srcElement;
	}
	
	var srcElement = currentElement;
	
	this.contextMenu = this.doc.createElement("div");
	this.contextMenu.className = "contextmenu";
	this.contextMenu.onclick = this.callbackCreator("contextMenuOnClick", null);
	
	var hasTable = false; // je mag maar 1 table in je context menu hebben
	
	while (currentElement && currentElement !== this)
	{
		switch (currentElement.nodeName.toLowerCase())
		{
			case "a":
				if (this.contextMenu.hasChildNodes())
				{
					this.addContextMenuSeparator();
				}
				this.addContextMenuItem(this.lang.cmenu_editLink, "editLink", currentElement, "img/link.png");
				break;
			case "img":
				if (this.contextMenu.hasChildNodes())
				{
					this.addContextMenuSeparator();
				}
				this.addContextMenuItem(this.lang.cmenu_editImage, "editImage", currentElement, "img/photo.png");
				break;
			case "table":
				if (hasTable) break;
				if (this.contextMenu.hasChildNodes())
				{
					this.addContextMenuSeparator();
				}
				this.addContextMenuItem(this.lang.cmenu_deleteTable, "deleteTable", currentElement, "img/table_delete.png");
				this.addContextMenuSeparator();
				
				if (currentElement.caption == null)
				{
					this.addContextMenuItem(this.lang.cmenu_addCaption, "addCaption", currentElement);
				}
				else
				{
					this.addContextMenuItem(this.lang.cmenu_deleteCaption, "deleteCaption", currentElement);
				}
				// Thead
				if (currentElement.tHead == null)
				{
					this.addContextMenuItem(this.lang.cmenu_addThead, "addTHead", currentElement);
				}
				else
				{
					this.addContextMenuItem(this.lang.cmenu_deleteThead, "deleteTHead", currentElement);
				}
				// Tfoot
				if (currentElement.tFoot == null)
				{
					this.addContextMenuItem(this.lang.cmenu_addTfoot, "addTFoot", currentElement);
				}
				else
				{
					this.addContextMenuItem(this.lang.cmenu_deleteTfoot, "deleteTFoot", currentElement);
				}
				
				var td = srcElement;
				while (td && td.nodeName.toLowerCase() != "td")
				{
					td = td.parentNode;
				}
				if (td && td.nodeName.toLowerCase() == "td")
				{
					if (this.contextMenu.hasChildNodes())
					{
						this.addContextMenuSeparator();
					}
					this.addContextMenuItem(this.lang.cmenu_deleteColumn, "deleteColumn", {"table" : currentElement, "td" : td});
					this.addContextMenuItem(this.lang.cmenu_addColumnAfter, "appendColumn", {"table" : currentElement, "td" : td});
					this.addContextMenuItem(this.lang.cmenu_addColumnBefore, "prependColumn", {"table" : currentElement, "td" : td});
					
					var tr = td.parentNode;
					while (tr && tr.nodeName.toLowerCase() != "tr")
					{
						tr = tr.parentNode;
					}
					if (tr && tr.nodeName.toLowerCase() == "tr")
					{
						var tbody = tr.parentNode;
						while (tbody && tbody.nodeName.toLowerCase() != "tbody")
						{
							tbody = tbody.parentNode;
						}
						if (tbody && tbody.nodeName.toLowerCase() == "tbody")
						{
							if (this.contextMenu.hasChildNodes())
							{
								this.addContextMenuSeparator();
							}
							this.addContextMenuItem(this.lang.cmenu_deleteRow, "deleteRow", {"tbody" : tbody, "tr" : tr});
							this.addContextMenuItem(this.lang.cmenu_addRowAfter, "appendRow", {"tbody" : tbody, "tr" : tr});
							this.addContextMenuItem(this.lang.cmenu_addRowBefore, "prependRow", {"tbody" : tbody, "tr" : tr});
						}
					}
				}
				hasTable = true;
				break;
		}
		
		currentElement = currentElement.parentNode;
	}
	
	if (!this.contextMenu.hasChildNodes())
	{
		this.contextMenu = null;
		return;
	}
	
	//this.doc.documentElement.appendChild(this.contextMenu); --> werkt niet in IE7
	this.doc.body.appendChild(this.contextMenu);
	var scroll_left = this.doc.documentElement.scrollLeft || this.doc.body.scrollLeft;
	var scroll_top = this.doc.documentElement.scrollTop || this.doc.body.scrollTop;
	this.contextMenu.style.top = (e.clientY + scroll_top) + "px";
	this.contextMenu.style.left = (e.clientX + scroll_left) + "px";
	
};

htmlEditor.prototype.addContextMenuSeparator = function()
{
	var d = this.doc.createElement("div");
	d.className = "menuseparator";
	this.contextMenu.appendChild(d);
};

htmlEditor.prototype.addContextMenuItem = function(txt, cbName, cbArg, img)
{
	var a = this.doc.createElement("a");
	if (img) a.style.backgroundImage = "url(" + img + ")";
	a.setAttribute("href", "javascript:void(0)");
	a.appendChild(this.doc.createTextNode(txt));
	a.onclick = this.callbackCreator(cbName, cbArg);
	this.contextMenu.appendChild(a);
};

htmlEditor.prototype.contextMenuOnClick = function(arg)
{
	this.closeContextMenu();
};

htmlEditor.prototype.closeContextMenu = function()
{
	if (this.contextMenu)
	{
		this.contextMenu.parentNode.removeChild(this.contextMenu);
		this.contextMenu = null;
	}
};

htmlEditor.prototype.deleteRow = function(arg)
{
	arg.tbody.deleteRow(arg.tr.sectionRowIndex);
};

htmlEditor.prototype.insertRow = function(tbody, tr, index)
{
	var colCount = tr.cells.length;
	var newRow = tbody.insertRow(index);
	for (var i = 0; i < colCount; i++)
	{
		var cell = newRow.insertCell(i);
		if (htmlEditor.is_firefox)
			cell.innerHTML = "<br />";
	}
};

htmlEditor.prototype.appendRow = function(arg)
{
	this.insertRow(arg.tbody, arg.tr, arg.tr.sectionRowIndex + 1);
};

htmlEditor.prototype.prependRow = function(arg)
{
	this.insertRow(arg.tbody, arg.tr, arg.tr.sectionRowIndex);
};

htmlEditor.prototype.appendColumn = function(arg)
{
	this.insertColumn(arg.table, arg.td, arg.td.cellIndex + 1);		
};

htmlEditor.prototype.insertColumn = function(table, td, index)
{
	for (var i = 0; i < table.rows.length; i++)
	{
		var cell = table.rows[i].insertCell(index);
		if (htmlEditor.is_firefox)
			cell.innerHTML = "<br />";
	}
	
	if (table.tHead)
	{
		var th = this.doc.createElement("th");
		if (htmlEditor.is_firefox)
			th.innerHTML = "<br />";
		table.tHead.rows[0].replaceChild(th, table.tHead.rows[0].cells[index]);
	}
	
	if (table.tFoot)
	{
		var th = this.doc.createElement("th");
		if (htmlEditor.is_firefox)
			th.innerHTML = "<br />";
		table.tFoot.rows[0].replaceChild(th, table.tFoot.rows[0].cells[index]);
	}
};

htmlEditor.prototype.prependColumn = function(arg)
{
	this.insertColumn(arg.table, arg.td, arg.td.cellIndex);
};

htmlEditor.prototype.deleteColumn = function(arg)
{
	var index = arg.td.cellIndex; // save for IE
	for (var i = 0; i < arg.table.rows.length; i++)
	{
		arg.table.rows[i].deleteCell(index);
	}
};

htmlEditor.prototype.addTHead = function(arg)
{
	var colCount = arg.rows[0].cells.length;
	var thead = arg.createTHead();
	var row = thead.insertRow(0);
	for (var i = 0; i < colCount; i++)
	{
		var th = this.doc.createElement("th");
		if (htmlEditor.is_firefox)
			th.innerHTML = "<br />";
		row.appendChild(th);
	}
};

htmlEditor.prototype.deleteTHead = function(arg)
{
	arg.deleteTHead();
};

htmlEditor.prototype.addTFoot = function(arg)
{
	var colCount = arg.rows[0].cells.length;
	var thead = arg.createTFoot();
	var row = thead.insertRow(0);
	for (var i = 0; i < colCount; i++)
	{
		var th = this.doc.createElement("th");
		if (htmlEditor.is_firefox)
			th.innerHTML = "<br />";
		row.appendChild(th);
	}
};

htmlEditor.prototype.deleteTFoot = function(arg)
{
	arg.deleteTFoot();
};

htmlEditor.prototype.deleteTable = function(arg)
{
	var win = this.doc.defaultView || this.doc.parentWindow;
	if (win.confirm(this.lang.confirm_deleteTable)) arg.parentNode.removeChild(arg);
};

htmlEditor.prototype.addCaption = function(arg)
{
	var value = prompt(this.lang.prompt_caption, "");
	if (value)
	{
		var caption = arg.createCaption();
		caption.appendChild(this.doc.createTextNode(value));
	}
};

htmlEditor.prototype.deleteCaption = function(arg)
{
	arg.deleteCaption();
};

htmlEditor.prototype.onKeyUp = function()
{
	this.updateStatus();
};

htmlEditor.prototype.onFocus = function()
{
	this.updateStatus();
};

htmlEditor.prototype.onMouseUp = function()
{
	this.closeContextMenu();
	this.updateStatus();
};

htmlEditor.prototype.updateStatus = function()
{
	var parent = this.getRangeParent();
	// In FF als je maximaliseert (position = absolute???) dan krijg je bij een klik in de div meteen als parent de editor zelf...
	if (parent === this.editor)
	{
		return;
	}
	
	for (var i = 0; i < this.toolbar.childNodes.length; i++)
	{
		var el = this.toolbar.childNodes[i];
		el.bindedElement = null;
		if (el.nodeName.toLowerCase() == "select")
		{
			el.selectedIndex = 0;
		}
		else if (el.tag)
		{
			el.className = "";
		}
	}
	
	while (this.statusbar.hasChildNodes())
	{
		if (this.statusbar.lastChild === this.statusbar.firstChild) break;
		this.statusbar.removeChild(this.statusbar.lastChild);
	}

	var elements = [];
	
	while (parent && parent !== this.editable)
	{
		var parentName = parent.nodeName.toLowerCase();
		
		elements.push(parent);
		
		// Toolbar
		for (var i = 0; i < this.toolbar.childNodes.length; i++)
		{
			var toolbarElement = this.toolbar.childNodes[i];
			
			switch (toolbarElement.nodeName.toLowerCase())
			{
				case "a":
					if (toolbarElement.tag && toolbarElement.tag == parentName)
					{
						toolbarElement.bindedElement = parent;
						toolbarElement.className = "activeButton";
					}
					break;
				case "select":
					if (toolbarElement.selectedIndex > 0)
					{
						break;
					}
					for (var j = 1; j < toolbarElement.options.length; j++)
					{
						var option = toolbarElement.options[j];
						
						if (option.$tagName && option.$tagName == parentName)
						{
							if (option.$attributeName)
							{
								// IE 7 en kleiner kunnen niet omgaan met getAttribute i.c.m. class, zij werken met className.
								var attrName = (option.$attributeName == "class" && typeof document.documentElement.hasAttribute == "undefined") ? "className" : "class";
								if (parent.getAttribute(attrName) == option.value)
								{
									toolbarElement.bindedElement = parent;
									toolbarElement.selectedIndex = j;
								}
							}
							else
							{
								toolbarElement.bindedElement = parent;
								toolbarElement.selectedIndex = j;
							}
						}
					}
					break;
			}
		}
		parent = parent.parentNode;
	}
	
	// Statusbar
	elements.reverse();
	for (var i = 0; i < elements.length; i++)
	{
		var arrow = this.doc.createElement("span");
		arrow.appendChild(this.doc.createTextNode(">"));
		this.statusbar.appendChild(arrow);
		
		var sa = this.doc.createElement("a");
		sa.onclick = this.callbackCreator("selectElement", {"statusbarItem" : sa, "element" : elements[i]});
		sa.setAttribute("href", "javascript:void(0)");
		sa.appendChild(this.doc.createTextNode(elements[i].nodeName.toLowerCase()));
		this.statusbar.appendChild(sa);
	}
};

htmlEditor.prototype.selectElement = function(arg)
{
	this.selectRange(arg.element);
};

htmlEditor.userAgent = navigator.userAgent.toLowerCase();
htmlEditor.is_ie = htmlEditor.userAgent.indexOf("msie") > -1;
htmlEditor.is_firefox = htmlEditor.userAgent.indexOf("firefox") > -1;
htmlEditor.is_safari = htmlEditor.userAgent.indexOf("safari") > -1;

htmlEditor.replaceTextArea = function(textarea, cssClassElements, imgClassElements, lang)
{
	var ed = new htmlEditor(textarea.ownerDocument, cssClassElements, imgClassElements, lang);
	
	var form = textarea.parentNode;
	while (form && form.nodeName.toLowerCase() != "form")
	{
		form = form.parentNode;
	}
	if (form && form.nodeName.toLowerCase() == "form")
	{
		var inpHidden = ed.doc.createElement("input");
		inpHidden.setAttribute("type", "hidden");
		inpHidden.setAttribute("name", textarea.getAttribute("name"));
		ed.inputHidden = inpHidden;
		form.appendChild(inpHidden);
		form.onsubmit = function()
		{
			return ed.onSubmit();
		};
	}
	
	ed.editable.style.height = (textarea.offsetHeight - 60) + "px";
	// Bug in FF: als er niets in de editable staat, en je klikt hier 2 maal achter elkaar in,
	// dan kun je niet meer editen.
	if (htmlEditor.is_firefox)
		ed.editable.innerHTML = textarea.value || "<br />";
	else
		ed.editable.innerHTML = textarea.value;
	
	ed.editor.style.width = (textarea.offsetWidth - 2) + "px";
	textarea.parentNode.replaceChild(ed.editor, textarea);
	
	return ed;
};

htmlEditor.prototype.onSubmit = function()
{
	this.inputHidden.setAttribute("value", this.editable.innerHTML);
	return true;
};

htmlEditor.prototype.toggleFullscreen = function(arg)
{	
	if (this.isFullscreen)
	{
		this.editor.style.position = "";
		this.editable.style.height = this.oldHeight;
		arg.className = "";
	}
	else
	{
		this.oldHeight = this.editable.style.height;
		this.editor.style.position = "absolute";
		this.editor.style.top = "10px";
		this.editable.style.height = (this.doc.documentElement.clientHeight - 80) + "px";
		arg.className = "activeButton";
	}
	this.isFullscreen = !this.isFullscreen;
};

htmlEditor.prototype.toggleWYSIWYG = function(arg)
{
	if (this.wysiwygMode)
	{
		var html = this.editable.innerHTML;
		
		this.editable.style.fontFamily = "Consolas";
		
		if (this.doc.createRange)
		{
			var contentNode = this.doc.createTextNode(html);
			this.editable.innerHTML = "";
			this.editable.appendChild(contentNode);
		}
		else
		{
			this.editable.innerText = html;
		}
		
		for (var i = 1; i < this.toolbar.childNodes.length; i++)
		{
			this.toolbar.childNodes[i].style.visibility = "hidden";
		}
		for (var i = 0; i < this.statusbar.childNodes.length; i++)
		{
			this.statusbar.childNodes[i].style.visibility = "hidden";
		}
		arg.className = "activeButton";
		
		this.editable.onkeyup = null;
		this.editable.onmouseup = null;
		this.editable.onfocus = null;
		this.editable.oncontextmenu = null;
	}
	else
	{
		this.editable.style.fontFamily = "";
		
		if (this.doc.createRange)
		{
			var r = this.doc.createRange();
			r.selectNodeContents(this.editable);
			this.editable.innerHTML = r.toString();
		}
		else
		{
			this.editable.innerHTML = this.editable.innerText;
		}
		for (var i = 1; i < this.toolbar.childNodes.length; i++)
		{
			this.toolbar.childNodes[i].style.visibility = "";
		}
		for (var i = 0; i < this.statusbar.childNodes.length; i++)
		{
			this.statusbar.childNodes[i].style.visibility = "";
		}
		arg.className = "";
		this.editable.onkeyup = this.callbackCreator("onKeyUp", null);
		this.editable.onmouseup = this.callbackCreator("onMouseUp", null);
		this.editable.onfocus = this.callbackCreator("onFocus", null);
		var me = this;
		this.editable.oncontextmenu = function(e)
		{
			me.onContextMenu(e);
		};
	}
	
	this.wysiwygMode = !this.wysiwygMode;
};

htmlEditor.prototype.executeBold = function(arg)
{
	this.surroundOrExtract("strong", arg);
};

htmlEditor.prototype.surroundOrExtract = function(cmd, btn)
{
	if (btn.className == "activeButton")
	{
		this.extractFromRange(btn.bindedElement);
	}
	else
	{
		if (this.isRangeCollapsed())
		{
			//alert("Er is geen selectie.");
			return;
		}
		this.surroundRange(cmd, null);
	}
};

htmlEditor.prototype.executeItalic = function(arg)
{
	this.surroundOrExtract("em", arg);
};

htmlEditor.prototype.executeListBullets = function(arg)
{
	var scrollpos = this.saveScrollForIe();
	this.editable.focus();
	this.doc.execCommand("InsertUnorderedList", false, null);
	this.restoreScroll(scrollpos);
};

htmlEditor.prototype.executeListNumbers = function()
{
	var scrollpos = this.saveScrollForIe();
	this.editable.focus();
	this.doc.execCommand("InsertOrderedList", false, null);
	this.restoreScroll(scrollpos);
};

// This is a fix for IE only: when this.editable.focus() is called,
// IE scrolls to the top of the editable...
htmlEditor.prototype.saveScrollForIe = function()
{
	if (!htmlEditor.is_ie)
		return null;

	return {
		"x" : window.pageXOffset ? window.pageXOffset : document.documentElement.scrollLeft ? document.documentElement.scrollLeft : document.body.scrollLeft,
		"y" : window.pageYOffset ? window.pageYOffset : document.documentElement.scrollTop ? document.documentElement.scrollTop : document.body.scrollTop
	};
};

htmlEditor.prototype.restoreScroll = function(scrollpos)
{
	if (scrollpos == null)
		return;

	var t = setTimeout(function()
	{
		window.scrollTo(scrollpos.x, scrollpos.y);
		clearTimeout(t);
	}, 0);
};

htmlEditor.prototype.showSpecialCharsWindowFunction = function()
{
	var o = {
		"lang" : this.lang
	};
	var result = window.showModalDialog("specialchars.html", o, "center:yes; scroll:yes; resizable:yes; dialogwidth:240px; dialogheight:400px; scroll:no; resizable:no;");
	if (result)
	{
		// IE fix: when editable.focus() is called, IE scrolls to the top of the editable element.
		// So when in-page editing, this is irritating if you edit something at the bottom of the page
		// and then IE scrolls to the top.
		// We MUST set the focus for IE here when the range is collapsed
		// So now we save the scroll position for IE en the do our thing, and after that,
		// we scroll back.
		var scrollpos = this.saveScrollForIe();

		if (!htmlEditor.is_safari)
			this.editable.focus(); // causes ie to scroll to top of editable.
		
		this.replaceRange(this.doc.createTextNode(result));
		
		this.restoreScroll(scrollpos);
			
	}
};

htmlEditor.prototype.showSpecialCharsWindow = function()
{
	if (htmlEditor.userAgent.indexOf("safari") > -1 && htmlEditor.userAgent.indexOf("chrome") == -1)
	{
		var me = this;
		var t = setTimeout(function()
		{
			me.showSpecialCharsWindowFunction();
			clearTimeout(t);
		}, 0);
	}
	else
	{
		this.showSpecialCharsWindowFunction();
	}
};

htmlEditor.prototype.prepareTableFunction = function()
{
	var o = {
		"lang" : this.lang
	};
	var result = window.showModalDialog("table.html", o, "center:yes; scroll:yes; resizable:yes; dialogwidth:400px; dialogheight:400px; scroll:no; resizable:no;");
	if (result)
	{
		var scrollpos = this.saveScrollForIe();
		// PAS OP:
		// 1) Nodig voor IE als je een lege selectie hebt.
		// 2) Voor FF maakt het niets uit.
		// 3) Voor Safari mag dit NOOIT!
		if (htmlEditor.userAgent.indexOf("safari") == -1)
			this.editable.focus();
		
		this.executeTable(result.rows, result.cols);
		
		this.restoreScroll(scrollpos);
	}
};

htmlEditor.prototype.prepareTable = function(arg)
{
	if (htmlEditor.userAgent.indexOf("safari") > -1 && htmlEditor.userAgent.indexOf("chrome") == -1)
	{
		var me = this;
		var t = setTimeout(function()
		{
			me.prepareTableFunction();
			clearTimeout(t);
		}, 0);
	}
	else
	{
		this.prepareTableFunction();
	}
};

htmlEditor.prototype.executeTable = function(rows, cols)
{
	var table = this.doc.createElement("table");
	var tbody = this.doc.createElement("tbody");
	
	for (var i = 0; i < rows; i++)
	{
		var tr = this.doc.createElement("tr");
		for (var j = 0; j < cols; j++)
		{
			var td = this.doc.createElement("td");
			if (htmlEditor.is_firefox)
				td.innerHTML = "<br />";
			tr.appendChild(td);
		}
		tbody.appendChild(tr);
	}
	table.appendChild(tbody);
	this.replaceRange(table);
};

htmlEditor.prototype.editLinkFunction = function(linkElement)
{
	var result = null;
	if (htmlEditor.is_firefox)
	{
		result = window.showModalDialog("link.html", {"link" : linkElement, "lang" : this.lang}, "center:yes; scroll:yes; resizable:yes; dialogwidth:400px; dialogheight:200px; scroll:no; resizable:no;");
	}
	else
	{
		result = window.showModalDialog("popup.html", {"link" : linkElement, "lang" : this.lang, "page" : "link.html"}, "center:yes; scroll:yes; resizable:yes; dialogwidth:400px; dialogheight:200px; scroll:no; resizable:no;");
	}
	if (result)
	{
		linkElement.setAttribute("href", result.href);
		if (result.title) linkElement.setAttribute("title", result.title);
		else
		{
			linkElement.removeAttribute("title");
		}
		
		if (result.embedprovider)
		{
			linkElement.className = result.embedprovider + "-oembed-link";
		}
		else
		{
			linkElement.removeAttribute("class");
			linkElement.removeAttribute("id");
		}
	}
};

htmlEditor.prototype.editLink = function(linkElement)
{
	if (htmlEditor.userAgent.indexOf("safari") > -1 && htmlEditor.userAgent.indexOf("chrome") == -1)
	{
		var me = this;
		var t = setTimeout(function()
		{
			me.editLinkFunction(linkElement);
			clearTimeout(t);
		}, 0);
	}
	else
	{
		this.editLinkFunction(linkElement);
	}
};

htmlEditor.prototype.prepareLinkFunction = function()
{
	// Alle browsers behalve FF laad ik in een popup frame: Chrome en Safari omdat je anders niet kunt navigeren in de popup zonder de returnvalue kwijt te raken
	// en IE omdat je scroll en resizable op NO moet zetten, maar dat je daardoor als de interne link klikt, je niet verder kunt scrollen.
	var result = null;
	if (htmlEditor.userAgent.indexOf("firefox") > -1)
	{
		result = window.showModalDialog("link.html", {"lang" : this.lang}, "center:yes; dialogwidth:400px; dialogheight:200px; scroll:no; resizable:no;");
	}
	else
	{
		result = window.showModalDialog("popup.html", {"lang" : this.lang, "page" : "link.html"}, "center:yes; dialogwidth:400px; dialogheight:200px; scroll:no; resizable:no;");
	}
	if (result)
	{
		var className = null;
		
		if (result.embedprovider)
		{
			className = result.embedprovider + "-oembed-link";
			delete result.embedprovider;
		}
		
		this.surroundRange("a", result, className);
	}
	
};

htmlEditor.prototype.prepareLink = function(arg)
{
	if (arg.className == "activeButton")
	{
		this.extractFromRange(arg.bindedElement);
	}
	else
	{
		if (this.isRangeCollapsed())
		{
			return;
		}
		if (htmlEditor.userAgent.indexOf("safari") > -1 && htmlEditor.userAgent.indexOf("chrome") == -1)
		{
			var me = this;
			var t = setTimeout(function()
			{
				me.prepareLinkFunction();
				clearTimeout(t);
			}, 0);
		}
		else
		{
			this.prepareLinkFunction();
		}
	}
};

htmlEditor.prototype.editImageFunction = function(imageElement)
{
	var ob = {
		"img" : imageElement,
		"imgClassElements" : this.imgClassElements,
		"lang" : this.lang
	};
	var result = window.showModalDialog("image.html", ob, "center:yes; dialogwidth:400px; dialogheight:340px; scroll:no; resizable:no;");
	
	if (result)
	{
		if (typeof imageElement.style.cssFloat != "undefined") imageElement.style.cssFloat = result.floatValue; // FF
		else imageElement.style.styleFloat = result.floatValue; // IE
		
		if (!imageElement.getAttribute("style")) imageElement.removeAttribute("style");
		if (result.title) imageElement.setAttribute("title", result.title);
		else imageElement.removeAttribute("title");
		
		if (result.imgClass) imageElement.className = result.imgClass;
		else typeof this.doc.documentElement.hasAttribute == "undefined" ? imageElement.removeAttribute("className") : imageElement.removeAttribute("class");
	}
};

htmlEditor.prototype.editImage = function(imageElement)
{
	if (htmlEditor.userAgent.indexOf("safari") > -1 && htmlEditor.userAgent.indexOf("chrome") == -1)
	{
		var me = this;
		var t = setTimeout(function()
		{
			me.editImageFunction(imageElement);
			clearTimeout(t);
		}, 0);
	}
	else
	{
		this.editImageFunction(imageElement);
	}
};

// Aparte functie gemaakt, want voor Safari moet je dit aanroepen vanuit een timeout.
htmlEditor.prototype.prepareImageFunction = function()
{
	var result = null;
	if (htmlEditor.userAgent.indexOf("safari") == -1)
	{
		result = window.showModalDialog("media.php?editor=true&type=image", null, "center:yes; scroll:yes; resizable:yes; dialogwidth:600px; dialogheight:400px");
	}
	else
	{
		result = window.showModalDialog("popup.html", {"page" : "media.php?editor=true&type=image"}, "center:yes; scroll:yes; resizable:yes; dialogwidth:600px; dialogheight:400px");
	}
	if (result)
	{
		var scrollpos = this.saveScrollForIe();
		
		if (htmlEditor.userAgent.indexOf("safari") == -1)
			this.editable.focus();
		
		var img = this.doc.createElement("img");
		img.setAttribute("src", result);
		this.replaceRange(img);
		
		this.restoreScroll(scrollpos);
	}
};

htmlEditor.prototype.prepareImage = function(arg)
{
	// Deze timeout is alleen nodig voor safari (NIET voor Chrome...)
	// Deze timeout zorgt er ook voor dat ze in Chrome en Safari door de popup blocker worden tegengehouden...
	// Dus alleen met timeout aanroepen als het Safari is.
	if (htmlEditor.userAgent.indexOf("safari") > -1 && htmlEditor.userAgent.indexOf("chrome") == -1)
	{
		var me = this;
		var t = setTimeout(function()
		{
			me.prepareImageFunction();
			clearTimeout(t);
		}, 0);
	}
	else
	{
		this.prepareImageFunction();
	}
};

htmlEditor.prototype.prepareDocumentFunction = function()
{
	var result = null;
	if (htmlEditor.userAgent.indexOf("safari") == -1)
	{
		result = window.showModalDialog("media.php?editor=true&type=document", null, "center:yes; scroll:yes; resizable:yes; dialogWidth:600px; dialogHeight:400px");
	}
	else
	{
		result = window.showModalDialog("popup.html", {"page" : "media.php?editor=true&type=document"}, "center:yes; scroll:yes; resizable:yes; dialogWidth:600px; dialogHeight:400px");
	}
	if (result)
	{
		this.surroundRange("a", {"href" : result});
	}
};

htmlEditor.prototype.prepareDocument = function(arg)
{
	if (this.isRangeCollapsed())
	{
		return;
	}
	if (htmlEditor.userAgent.indexOf("safari") > -1 && htmlEditor.userAgent.indexOf("chrome") == -1)
	{
		var me = this;
		var t = setTimeout(function()
		{
			me.prepareDocumentFunction();
			clearTimeout(t);
		}, 0);
	}
	else
	{
		this.prepareDocumentFunction();
	}
};

htmlEditor.prototype.executeSelect = function(select)
{
	var option = select.options[select.selectedIndex];
	
	if (select.bindedElement === null)
	{
		if (this.isRangeCollapsed())
		{
			//alert("Er is geen selectie.");
			return;
		}
		if (option.$tagName)
		{
			var attributes = null;
			if (option.$attributeName)
			{
				attributes = new Object();
				attributes[option.$attributeName] = option.value;
			}
			this.surroundRange(option.$tagName, attributes);
			select.selectedIndex = 0;
		}
	}
	else
	{
		if (select.selectedIndex == 0)
		{
			this.extractFromRange(select.bindedElement);
		}
		else
		{
			var attributes = null;
			if (option.$attributeName)
			{
				attributes = new Object();
				attributes[option.$attributeName] = option.value;
			}
			this.switchElement(select.bindedElement, option.$tagName, attributes);
		}
	}
};

htmlEditor.prototype.executeRemoveFormat = function(arg)
{
	this.doc.execCommand("removeFormat", false, null);
};

// Nieuwe code die werkt met de Range DOM, i.p.v. alleen met execCommand.
htmlEditor.prototype.isRangeCollapsed = function()
{
	if (this.doc.defaultView && this.doc.defaultView.getSelection)
	{
		return this.doc.defaultView.getSelection().getRangeAt(0).collapsed;
	}
	else
	{
		return this.doc.selection.type == "Control" ? false : this.doc.selection.createRange().text.length == 0;
	}
};

htmlEditor.prototype.switchElement = function(oldElement, newTagName, attributes)
{
	if (this.doc.defaultView && this.doc.defaultView.getSelection)
	{
		var range = this.doc.defaultView.getSelection().getRangeAt(0);
		range.selectNodeContents(oldElement);
		var docFrag = range.extractContents();
		var newEl = this.doc.createElement(newTagName);
		for (var key in attributes)
		{
			if (attributes[key]) newEl.setAttribute(key, attributes[key]);
		}
		newEl.appendChild(docFrag);
		var parent = oldElement.parentNode;
		parent.replaceChild(newEl, oldElement);
		//parent.normalize();
		//r.collapse(false);
		//return newEl;
		this.doc.defaultView.getSelection().collapse(newEl, true);
	}
	else
	{
		var s = newTagName;
		for (var key in attributes)
		{
			if (attributes[key]) s += " " + key + "=\"" + attributes[key] + "\"";
		}
		oldElement.outerHTML = "<" + s + ">" + oldElement.innerHTML + "</" + newTagName + ">";
	}
};

htmlEditor.prototype.extractFromRange = function(element)
{
	if (this.doc.defaultView && this.doc.defaultView.getSelection)
	{
		var range = this.doc.defaultView.getSelection().getRangeAt(0);
		range.selectNodeContents(element);
		var docFrag = range.extractContents();
		var parent = element.parentNode;
		parent.replaceChild(docFrag, element);
		//parent.normalize();
		//return parent;
	}
	else
	{
		element.outerHTML = element.innerHTML;
	}

};

htmlEditor.prototype.getRangeParent = function()
{
	if (this.doc.defaultView && this.doc.defaultView.getSelection)
	{
		if (this.doc.defaultView.getSelection().rangeCount == 0)
		{
			// Safari...
			return null;
		}
		return this.doc.defaultView.getSelection().getRangeAt(0).commonAncestorContainer;
	}
	else
	{
		var range = this.doc.selection.createRange();
		return this.doc.selection.type == "Control" ? range.item(0) : range.parentElement();
	}
};

htmlEditor.prototype.replaceRange = function(element)
{
	if (this.doc.defaultView && this.doc.defaultView.getSelection)
	{
		var range = this.doc.defaultView.getSelection().getRangeAt(0);
		range.deleteContents();
		range.insertNode(element);
		element.parentNode.normalize();
	}
	else
	{
		var range = this.doc.selection.createRange();
		if (this.doc.selection.type == "Control")
			range.item(0).parentNode.replaceChild(element, range.item(0))
		else
			range.pasteHTML(element.outerHTML || element.nodeValue); // een textNode heeft geen outerHTML...
		//range.select(); // in-page-ie-fix
	}
};

htmlEditor.prototype.surroundRange = function(tagName, attributes, className)
{
	if (this.doc.defaultView && this.doc.defaultView.getSelection)
	{
		var range = this.doc.defaultView.getSelection().getRangeAt(0);
		var element = this.doc.createElement(tagName);
		if (attributes)
		{
			for (var key in attributes)
			{
				if (attributes[key]) element.setAttribute(key, attributes[key]);
			}
			//if (id) element.id = id;
			if (className) element.className = className;
		}
		try
		{
			// surroundContents() geeft nu FF 3.5 een exxception als je een regel wil omgeven, terwijl
			// begin van de regel al door een bepaald element is omgeven, bijv.
			// <em>dit</em> is een test ==> als je hier van wil maken <p><em>Dit</em> is een test</p>
			// en je selecteert de hele tekst, dan krijg je een excerption...
			//range.surroundContents(element);
			element.appendChild(range.extractContents());
			range.insertNode(element);
		}
		catch (e)
		{
			alert(range.commonAncestorContainer.innerHTML);
			range.setStartBefore(range.commonAncestorContainer);
			range.surroundContents(element);
		}
		this.doc.defaultView.getSelection().collapse(element, true);
	}
	else
	{
		var range = this.doc.selection.createRange();
		var s = tagName;
		for (var key in attributes)
		{
			if (attributes[key]) s += " " + key + "=\"" + attributes[key] + "\"";
		}
		//if (id) s += " id=\"" + id + "\"";
		if (className)  s += " class=\"" + className + "\"";
		if (this.doc.selection.type == "Control")
		{
			var element = range.item(0);
			element.outerHTML = "<" + s + ">" + element.outerHTML + "</" + tagName + ">";
		}
		else
		{
			range.pasteHTML("<" + s + ">" + range.htmlText + "</" + tagName + ">");
		}
	}
};

htmlEditor.prototype.selectRange = function(element)
{
	if (this.doc.defaultView && this.doc.defaultView.getSelection)
	{
		//~ // Onderstaande regel is nodig, anders blijf je in FF met een geblurde selectie zitten
		this.doc.defaultView.getSelection().collapseToStart();
		var range = this.doc.defaultView.getSelection().getRangeAt(0);
		range.selectNode(element);
			
		// Voor opera en safari is dit nodig, anders zie je de selectie niet
		if (htmlEditor.userAgent.indexOf("safari") > -1)
		{
			var sel = this.doc.defaultView.getSelection();
			sel.collapse(element, 1);
			sel.collapse(element, element.childNodes.length);
			sel.addRange(range);
		}
	}
	else
	{
		var range = this.doc.selection.createRange();
		if (this.doc.selection.type == "Control")
			{
				range.select();
			}
			else
			{
				range.moveToElementText(element);
				range.select();
			}
	}
	
};
