<?php

class DbPage extends DbEntity
{
	/* Maps a database reader object to a Page object
	 * @obj: database reader object
	 * returns: Page object */
	protected function map($obj)
	{
		$page = new Page();
		$page->id = $obj->id;
		$page->parentid = $obj->parentid;
		$page->sortorder = $obj->sortorder;
		$page->menuname = $obj->menuname;
		$page->prepath = $obj->prepath;
		$page->pathname = $obj->pathname;
		$page->title = $obj->title;
		$page->depth = $obj->depth;
		$page->visible = $obj->visible;
		$page->locked = $obj->locked;
		$page->template = $obj->template;
		$page->collection_id = $obj->collection_id;
		$page->collection_limit = $obj->collection_limit;
		$page->cache_minutes = $obj->cache_minutes;
		$page->is_homepage = $obj->is_homepage;
		$page->meta_keywords = $obj->meta_keywords;
		$page->meta_description = $obj->meta_description;
		return $page;
	}
	
	/* Queries the database for pages the surround $page.
	 * Always gets the root pages (parentid 0) and fixed pages (parentid -1).
	 * The child and sibling pages depends on $strict:
	 * - if it is true, only the pages under each part of the prepath are queried
	 * - if it is false, all pages under the root prepath are queried
	 * Example: page = 'projects/private/myproject'
	 * -> $strict = true: all pages that have a prepath in (projects/, projects/private/ and projects/private/myproject/) are queried
	 * -> $strict = false: all pages that have a prepath like projects/% are queried.
	 * @page: the active page
	 * @strict: true or false, see above
	 * returns: an associative array:
	 * 	pages['ID'][id] => Page
	 *	pages['PARENT_ID'][parentid][id] => Page
	 * If no pages are found, the pages['ID'] and pages['PARENT_ID'] are empty */
	public function GetSurroundingVisible($page, $strict)
	{
		$pages = array('ID' => array(), 'PARENT_ID' => array());

		$prepath = $page->parentid <= 0 ? $page->pathname : substr($page->prepath, 0, strpos($page->prepath, '/'));
		$query = '';
		
		if ($strict)
		{
			$inString = '';
			$a = explode('/', $page->prepath);
			$sqlIn = array();
			foreach ($a as $s)
			{
				if (strlen($s) > 0)
				{
					$inString .= $s . '/';
					$sqlIn[] = "'$inString'";
				}
			}
			$sqlIn[] = '\'' . $page->prepath . $page->pathname . '/\'';
			$inString = implode(',', $sqlIn);
			$query = 'select p.*, t.`name` as template from pages as p, templates as t where (p.visible = 1) AND (p.parentid = -1 OR p.parentid = 0
				OR p.prepath in (' . $inString . '))
				order by p.depth, p.sortorder';
		}
		else
		{
			$query = 'select p.*, t.`name` as template from pages as p, templates as t where (p.visible = 1) AND
				(p.parentid = -1 OR p.parentid = 0 OR p.prepath like \'' . $prepath . '/%\')
				order by p.depth, p.sortorder';
		}

		if ($result = $this->mysqli->query($query))
		{
			while ($obj = $result->fetch_object())
			{
				$p = $this->map($obj);
				$ref =& $p;
				
				$pages['ID'][$p->id] = $ref;
				
				if (!isset($pages['PARENT_ID'][$p->parentid]))
				{
					$pages['PARENT_ID'][$p->parentid] = array();
				}
				$pages['PARENT_ID'][$p->parentid][$p->id] = $ref;
			}
		}
		
		return $pages;
	}
	
	/* Gets all visible pages
	 * returns: pages array (see GetSurroundingVisible) */
	public function GetAllVisible()
	{
		return $this->GetAll(true);
	}

	/* Gets all pages
	 * @onlyVisible: true for only visible, false for all pages
	 * returns: pages array (see GetSurroundingVisible) */
	protected function GetAll($onlyVisible)
	{
		$pages = array('ID' => array(), 'PARENT_ID' => array());
		
		$sqlVisible = $onlyVisible ? ' where p.visible = 1 ' : '';
		
		if ($result = $this->mysqli->query(
			'select p.*, t.`name` as template from pages as p, templates as t ' . $sqlVisible . ' order by p.parentid, p.sortorder'))
		{
			while ($obj = $result->fetch_object())
			{
				$p = $this->map($obj);
				$ref =& $p;
				
				$pages['ID'][$p->id] = $ref;
				
				if (!isset($pages['PARENT_ID'][$p->parentid]))
				{
					$pages['PARENT_ID'][$p->parentid] = array();
				}
				$pages['PARENT_ID'][$p->parentid][$p->id] = $ref;
			}
			$result->close();
		}
		
		return $pages;
	}
		
	/* Queries the database for a specific page with this id
	 * @id: the id of the page
	 * returns: a Page or null */
	public function GetById($id)
	{
		$page = null;
		
		if ($this->mysqli->multi_query(
			'select p.*, t.`name` as template from pages as p, templates as t where p.id = ' . $id . ' and p.templateid = t.id;
			select * from pages_content where page_id = ' . $id))
		{
			if ($result = $this->mysqli->use_result())
			{
				if ($obj = $result->fetch_object())
				{
					$page = $this->map($obj);
					$page->content = array();
				}
				$result->close();
				if ($page != null && $this->mysqli->next_result())
				{
					if ($result = $this->mysqli->use_result())
					{
						while ($obj = $result->fetch_object())
						{
							$page->content[$obj->content_id] = $obj->content;
						}
						$result->close();
					}
				}
			}
		}
		
		return $page;
	}

	/* By default, the first layered page is the homepage, but you can overrule it in the config table
	 * returns: the home Page object or null */
	public function GetHomePage()
	{
		$page = null;
		
		if ($this->mysqli->multi_query(
			'select p.*, t.`name` as template from pages as p, templates as t where p.is_homepage = 1 and p.templateid = t.id limit 1;
			select pc.* from pages_content as pc, pages as p where p.is_homepage = 1 and p.id = pc.page_id'))
		{
			if ($result = $this->mysqli->use_result())
			{
				if ($obj = $result->fetch_object())
				{
					$page = $this->map($obj);
					$page->content = array();
				}
				$result->close();
				if ($page != null && $this->mysqli->next_result())
				{
					if ($result = $this->mysqli->use_result())
					{
						while ($obj = $result->fetch_object())
						{
							$page->content[$obj->content_id] = $obj->content;
						}
						$result->close();
					}
				}
			}
		}
		
		return $page;
	}

	/* Queries the database for a specific page uniquely identified by prepath and pathname
	 * @prepath: the prepath (subfolders)
	 * @pathname: the pathname (name of the page)
	 * returns: a Page object or null */
	public function GetByPathInfo($prepath, $pathname)
	{
		if ($pathname == '')
		{
			return $this->GetHomePage();
		}

		$page = null;
		
		if ($this->mysqli->multi_query(
		'select p.*, t.`name` as template from pages as p, templates as t where p.prepath = \'' . $prepath . '\' and p.pathname = \'' . $pathname . '\' and p.templateid = t.id;
		select pc.* from pages_content as pc, pages as p where p.prepath = \'' . $prepath . '\' and p.pathname = \'' . $pathname . '\' and p.id = pc.page_id'))
		{
			if ($result = $this->mysqli->use_result())
			{
				if ($obj = $result->fetch_object())
				{
					$page = $this->map($obj);
					$page->content = array();
				}
				$result->close();
				if ($page != null && $this->mysqli->next_result())
				{
					if ($result = $this->mysqli->use_result())
					{
						while ($obj = $result->fetch_object())
						{
							$page->content[$obj->content_id] = $obj->content;
						}
						$result->close();
					}
				}
			}
		}
		
		return $page;
	}
	
	/* This method is called from the index.php file. It has all the logic for getting all the properties
	 * for showing a page.
	 * @page: Page object that you want to show
	 * @dbPage: DbPage object that is used to query the database
	 * @mysqli: a MySQLi object
	 * returns: a Template object */
	public static function PrepareToShow(Page $page, DbPage $dbPage, MySQLi $mysqli)
	{
		if (!file_exists(WE_PATH_ROOT . '/webessence/templates/' . $page->template))
		{
			$mysqli->close();
			die('Fout bij laden template: &#39;' . WE_PATH_ROOT . '/webessence/templates/' . $page->template . '&#39; bestaat niet.');
		}
		
		$tpl = new Template(WE_PATH_ROOT . '/webessence/templates/' . $page->template);
		$tpl->set('page', $page);
		
		$tpl->set('pages', WE_ALL_PAGES ? $dbPage->GetAllVisible() : $dbPage->GetSurroundingVisible($page, true));
		
		if ($page->collection_id > 0)
		{
			$dbCollectionItem = new DbCollectionItem($mysqli);
			$dbCollection = new DbCollection($mysqli);
			
			$tagname = empty($_GET['tag']) ? null : $_GET['tag'];
			
			$collection = $dbCollection->GetById($page->collection_id, $tagname);
			
			$tpl->set('collection', $collection);
			$tpl->set('tagscount', $dbCollection->GetTagCount($page->collection_id));
			
			// Nu kijken a.h.v. year, month, day en title of er een item of een collectie getoond moet worden
			if (isset($_GET['title']))
			{
				$item = $dbCollectionItem->GetItem($collection, $_GET['year'], $_GET['month'], $_GET['day'], $_GET['title']);
				$tpl->set('collectionitem', $item);
				if ($collection->has_comments)
				{
					$dbComment = new DbComment($mysqli);
					$tpl->set('comments', $dbComment->GetAccepted($item->id));
					if (!$item->comments_closed)
					{
						// Toevoegen om commentform spam-free te kunnen maken
						// Ik ga ervan uit dat een commentform een email, name, comment input heeft
						$time = time();
						$hash = sha1($time . $_SERVER['REMOTE_ADDR'] . $item->id . WE_SECRET);
						$tpl->set('hash', $hash);
						$tpl->set('timestamp', $time);
						
						$honeyPots = array();
						for ($i = 0; $i < 3; $i++)
							$honeyPots[] = sha1($hash . 'honeypot' . $i . WE_SECRET);
						for ($i = 0; $i < 3; $i++)
							$honeyPots[] = null;
						
						$tpl->set('email_field', sha1($hash . 'email' . WE_SECRET));
						$tpl->set('name_field', sha1($hash . 'name' . WE_SECRET));
						$tpl->set('comment_field', sha1($hash . 'comment' . WE_SECRET));
						$tpl->set('honeyPots', $honeyPots);
					}
				}
			}
			else if (isset($_GET['tag']))
			{
				$offset = isset($_GET['offset']) ? $_GET['offset'] : 0;
				$tpl->set('tag', $_GET['tag']);
				$tpl->set('offset', $offset);
				$tpl->set('collectionitems', $dbCollectionItem->GetItemsByTagname($collection, $_GET['tag'], $offset, $page->collection_limit));
			}
			else
			{
				$offset = isset($_GET['offset']) ? $_GET['offset'] : 0;
				$year = isset($_GET['year']) ? $_GET['year'] : null;
				$month = isset($_GET['month']) ? $_GET['month'] : null;
				$day = isset($_GET['day']) ? $_GET['day'] : null;
				$tpl->set('offset', $offset);
				$tpl->set('collectionitems', $dbCollectionItem->GetItems($collection, $year, $month, $day, $offset, $page->collection_limit));
			}
		}
		
		return $tpl;
	}
}

?>
